/* -*- c++ -*-  */
/* Cont fragments de codi de la practica de laboratori de la assignatura
   de Visualitzaci avanada del departament de LSI de la
   Factultad Informatica de Barcelona, de la UPC 2009-2010 */

#ifndef _MATERIAL_LIB_H_
#define _MATERIAL_LIB_H_

#include <vector>
#include <map>
#include <iostream>
#include <string>

using namespace std;

class Color {   
 public:
  Color( float red = 0.0f, float green = 0.0f, float blue = 0.0f, float alfa = 1.0f):
      r( red), g( green), b( blue), a( alfa) {};
  
  // atributs
  float r,g,b,a ;	// components RGB del color
};

// Material - permet representar les propietats optiques d'un material
class Material {
 public:
  Material( const string &_name = "default", 
           const Color &_ka = Color( 0.2f, 0.2f, 0.2f), 
           const Color &_kd = Color( 0.8f, 0.8f, 0.8f),
           const Color &_ks = Color( 1.0f, 1.0f, 1.0f),
           float _shininess = 10.f):
      name( _name), ka( _ka), kd( _kd), ks( _ks), shininess( _shininess) {};
  
  string name;
  Color ka, kd, ks;
  float shininess; 
};

class MaterialLib {
 public:
  MaterialLib();
  
  // llegeix els materials d'un fitxer .mtl, afegint-los a la biblioteca 
  void readMtl( const char *filename);
  
  // retorna el material corresponent a un index donat;
  // si no hi ha cap material amb aquest index, es retorna el material per defecte ( index 0)
  const Material &material( int index) const;
  
  // donat un nom de material, retorna el seu index
  int index( const string &name) const {
    return dict[ name];
  }
  
 private:
  vector<Material> materials;
  mutable map<string, int> dict;
  
  friend ostream &operator<<( ostream &os, const MaterialLib &m);
};



ostream &operator<<( ostream &os, const Color &c);
ostream &operator<<( ostream &os, const Material &m);
ostream &operator<<( ostream &os, const MaterialLib &m);

#endif
