/* -*- c++ -*-  */
/* Cont fragments de codi de la practica de laboratori de la assignatura
   de Visualitzaci avanada del departament de LSI de la
   Factultad Informatica de Barcelona, de la UPC 2009-2010 */

#ifndef _OBJECT_H_
#define _OBJECT_H_
#include <vector>

#include "Point.h"
#include "MaterialLib.h"

// Vertex - representa un vertex d'un objecte 3D
class Vertex {
 public:
  Vertex( const Point &coordenades): coord( coordenades) {};
  
  // atributs
  Point coord;		// coordenades del vertex
  Vector normal;  // vector normal al vèrtex ( per suavitzat)
};

// Box - permet representar una capsa aliniada amb els eixos
// ( ex. capsa englobant)
class Box {
 public:
  Box( const Point &minimum = Point(), const Point &maximum = Point()): minb( minimum), maxb( maximum) {}
  
  // update - expandeix la capsa per que inclogui el punt donat
  void update( const Point &p) {
    if ( p.x < minb.x) minb.x = p.x;
    if ( p.y < minb.y) minb.y = p.y;
    if ( p.z < minb.z) minb.z = p.z;
    if ( p.x > maxb.x) maxb.x = p.x;
    if ( p.y > maxb.y) maxb.y = p.y;
    if ( p.z > maxb.z) maxb.z = p.z;
  }
  
  // init - inicialitza la capsa amb el punt donat
  void init( const Point &p) {
    minb = p;
    maxb = p;
  }
  
  float diagonal2() { Vector dif = maxb - minb; return dif.squaredLength();}
  float diagonal() { Vector dif = maxb - minb; return dif.length();}
  
  Point Center() {
    return ( Point( ( maxb.x+minb.x)/2.0f,( maxb.y+minb.y)/2.0f,( maxb.z+minb.z)/2.0f));
  }
  
  // atributs
  Point minb, maxb;	// extrems mnim i mxim de la capsa
};

// Face - representa una cara d'un objecte 3D
class Face {
 public:
  Face() { 
    normal.x = 0.0f;
    normal.y = 0.0f;
    normal.z = 0.0f;
  }
  
  // constructor a partir de 3 o 4 indexs a vertex
  Face( int i1, int i2, int i3, int i4 = -1) {
    vertices.push_back( i1);
    vertices.push_back( i2);
    vertices.push_back( i3);
    if ( i4 != -1) vertices.push_back( i4);
    normal.x = 0.0f;
    normal.y = 0.0f;
    normal.z = 0.0f;
  }
  
  void updateNormal( vector<Vertex> &v) {
    size_t j;
    normal.x = 0.0f;
    normal.y = 0.0f;
    normal.z = 0.0f;
    size_t n = vertices.size();
    for ( size_t i = 0; i<n; i++) {
      j = ( i+1)%n;
      
      normal.x += ( ( v[ vertices[ i]].coord.z+v[ vertices[ j]].coord.z) *
                    ( v[ vertices[ i]].coord.y-v[ vertices[ j]].coord.y));
      normal.y += ( ( v[ vertices[ i]].coord.x+v[ vertices[ j]].coord.x) *
                    ( v[ vertices[ i]].coord.z-v[ vertices[ j]].coord.z));
      normal.z += ( ( v[ vertices[ i]].coord.y+v[ vertices[ j]].coord.y) *
                    ( v[ vertices[ i]].coord.x-v[ vertices[ j]].coord.x));
    }
    
    normal.x *= 0.5f;
    normal.y *= 0.5f;
    normal.z *= 0.5f;
    normal.normalize();
  }

  // atributs
  int material;	       // ID del material ( index del material)
  vector<int> vertices;  // vector amb els indexs dels vertexs de la cara
  Vector normal;
};

using namespace std;

// Object - permet representar un objecte 3D
class Object {
 public:
  Object( std::string);
  ~Object();
  
  // llegeix un model en format OBJ
  void readObj( const char *filename, MaterialLib &matlib);
  
  // calcula la capsa del model
  void updateBoundingBox();
  
  // retorna la capsa del model ( previament s'ha d'haver calculat)
  Box boundingBox() const;
  
  void updateNormals();
  void translate( float x, float y, float z);

  int write( const char *fileName); // write mesh in GiD ASCII format

  // para el modo Vertex Arrray
  void createVALists();
  void deleteVALists();
  void renderVALists();

  size_t getNumVertices() const { return _numVertices;}
  size_t getNumTriangs() const { return _numTriangs;}
  size_t getNumQuads() const { return _numQuads;}
  float *getVAVertices() const { return _lstVertices;}
  float *getVANormales() const { return _lstNormales;}
  unsigned int *getVATriangs() const { return _lstIndicesTriangs;}
  unsigned int *getVAQuads() const { return _lstIndicesQuads;}
  int getIdxMaterial() const { return faces[ 0].material;}

  const char *get_err_str() const { return _err_str;}
  void set_err_str( const char *str);

 private:
  void make_face ( char **words, int nwords, int material );
  void clear ();

 public:
  vector<Vertex> vertices;  // vector amb els vertexs de l'objecte
  vector<Face> faces;       // vector amb les cares de l'objecte

 private: 
  std::string name;
  Box _boundingBox;   // caixa contenidora de l'objecte.

  // para el modo Vertex Arrray
  size_t _numVertices, _numFaces;
  float *_lstVertices;
  float *_lstNormales;
  unsigned int *_lstIndicesTriangs;
  unsigned int *_lstIndicesQuads;
  int _numTriangs, _numQuads;
        
  // para puntos, normales, triangulos y cuadrados
  // tambien usaremos los datos de VA

  char *_err_str;
};


#endif
