#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef _WIN32
#define strdup _strdup
#endif

#define BUILD_GID_PLUGIN
#include "gid_plugin_import.h"
/* 
DECLARE_GID_PLUGIN_FUNCTIONS;
*/

#include "Object.h"
// #include "gid_arrays.h"
// 
// typedef struct {
//   float x, y, z;
// } _t_objVector3;
// 
// typedef struct {
//   float s, t;
// } _t_objVector2;
// 
// typedef struct {
//   int n1, n2, n3;
// } _t_objTriangle;
// 
// typedef struct {
//   int n1, n2, n3;
// } _t_objQuad;
// 
// GidDynamicArray< _t_objVector3> _G_lst_points( TRUE);
// GidDynamicArray< _t_objVector3> _G_lst_normals( TRUE);
// GidDynamicArray< _t_objVector2> _G_lst_texCoords( TRUE);
// GidDynamicArray< GidDynamicArray< _t_objTriangle>> _G_lst_triangles;
// GidDynamicArray< GidDynamicArray< _t_objQuad>> _G_lst_quads;

// #ifdef __linux__
// void __attribute__( ( constructor)) inicio() {
//   printf( "-->inicio libraria: %d\n", hola);
// 
// }
// 
// void __attribute__( ( destructor)) fin() {
//   printf( "-->fin libraria\n");
// }
// #elif defined (_WIN32)
// 
// #include <windows.h>
// 
// BOOL APIENTRY DllMain( HMODULE h_module,
//                        DWORD  ul_reason_for_call,
//                        LPVOID lp_reserved) {
//   char *str = "N/A";
//   switch ( ul_reason_for_call) {
//     case DLL_PROCESS_ATTACH:
//       str = "PROCESS ATTACH"; break;
//     case DLL_THREAD_ATTACH:
//       str = "THREAD ATTACH"; break;
//     case DLL_THREAD_DETACH:
//       str = "THREAD DETACH"; break;
//     case DLL_PROCESS_DETACH:
//       str = "PROCESS DETACH"; break;
//       break;
//   }
//   printf( "-->hello: %s - %d\n", str, hola);
//   return TRUE;
// }
// 
// #endif // __linux__

static char *_G_err_str = NULL;

void set_err_str( const char *str = NULL) {
  if ( _G_err_str) free( _G_err_str);
  _G_err_str = NULL;
  if ( str) {
    _G_err_str = strdup( str);
  }
}

int create_demo_triangs() {
  GiD_NewPostProcess();
  int mesh_id = GiD_NewMesh( GID_POST, GIDPOST_NEW_MESH, "Hello");
  GiD_SetColor( mesh_id, 0.5f, 0.3f, 0.8f, 0.0f);
  float lst_vertices[ 4][ 3] = { { 0.0, 0.0, 0.0}, { 1.0, 0.0, 0.0}, { 1.0, 1.0, 0.0}, { 0.0, 1.0, 0.0}};
  GiD_SetVertexPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_VERTICES, 3, 4, 0, lst_vertices);
  int lst_elements[ 2][ 3] = { { 0, 1, 2}, { 2, 3, 0}};
  GiD_SetElementPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_CONNECTIVITIES, GIDPOST_TRIANGLE, 2, 0, lst_elements, 0, NULL);
  
  float lst_scalar_res[ 4] = { 0.0, 1.0, 2.0, 3.0};
  int res_id = GiD_NewResult( "new analysis", 0.0, "test result", 0); // 0 --> for all meshes
  GiD_SetResultPointer( res_id, GIDPOST_FLOAT, GIDPOST_RESULTS, GIDPOST_SCALAR, GIDPOST_NODAL, 4, 0, lst_scalar_res);
  GiD_EndResult( res_id);
  GiD_EndMesh( mesh_id);
  set_err_str( "Everything fine.");
  return 0;
}

int obj_2_gid( const char *filename) {
  const char *name = strrchr( filename, '/');
#ifdef _WIN32
  if ( !name) name = strrchr( filename, '\\');
#endif // _WIN32
  if ( !name) name = filename;
  else name++;

  Object wf_obj( name);
  MaterialLib lib_mat;
  wf_obj.readObj( filename, lib_mat);
  wf_obj.createVALists(); // to create the arrays as we need to send to GiD

  int ret = 1;
  if ( wf_obj.getNumVertices() && ( wf_obj.getNumTriangs() ||wf_obj.getNumQuads())) {
    GiD_NewPostProcess();
    int mesh_id = GiD_NewMesh( GID_POST, GIDPOST_NEW_MESH, name);
    // GiD_SetColor( mesh_id, 0.5f, 0.3f, 0.8f, 0.0f);
    Material mat_col = lib_mat.material( wf_obj.getIdxMaterial());
    GiD_SetColor( mesh_id, mat_col.kd.r, mat_col.kd.g, mat_col.kd.b, mat_col.kd.a);
    
    GiD_SetVertexPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_VERTICES, 3,
                      ( int)wf_obj.getNumVertices(), 0, wf_obj.getVAVertices());

    if ( wf_obj.getNumTriangs()) {
      GiD_SetElementPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_CONNECTIVITIES, GIDPOST_TRIANGLE,
                         ( int)wf_obj.getNumTriangs(), 0, wf_obj.getVATriangs(), 0, NULL);
    }
    if ( wf_obj.getNumTriangs()) {
      GiD_SetElementPointer( mesh_id, GIDPOST_FLOAT, GIDPOST_CONNECTIVITIES, GIDPOST_QUADRILATERAL,
                         ( int)wf_obj.getNumQuads(), 0, wf_obj.getVAQuads(), 0, NULL);
    }
    
    int res_id = GiD_NewResult( "Geometry", 0.0, "Normals", 0); // 0 --> for all meshes
    GiD_SetResultPointer( res_id, GIDPOST_FLOAT, GIDPOST_RESULTS, GIDPOST_VECTOR_3, GIDPOST_NODAL,
                      ( int)wf_obj.getNumVertices(), 0, wf_obj.getVANormales());
    
    GiD_EndResult( res_id);
    GiD_EndMesh( mesh_id);
    if ( wf_obj.get_err_str() && *wf_obj.get_err_str()) {
      set_err_str( wf_obj.get_err_str());
    } else {
      set_err_str( "Everything fine.");
    }
    ret = 0;
  } else {
    if ( wf_obj.get_err_str() && *wf_obj.get_err_str()) {
      set_err_str( wf_obj.get_err_str());
    } else {
      set_err_str( "Nothing read.");
    }
    ret = 1;
  }
  return ret;
}

/*************************************************************************
 * Interface functions to GiD
 *************************************************************************/

// must be of type:
// typedef int ( *_t_fncGiD_PluginImport)( const char *filename);
// called by GiD
extern "C" GID_DLL_EXPORT int GiD_PostImportFile( const char *filename) {

  char *path2file = strdup( filename);
  char *s = strrchr( path2file, '/');
  if ( !s) s = strrchr( path2file, '\\');
  if ( s) *( ++s) = '\0';
  else strcpy( path2file, "./");
  
  bool res = true;
  int ret_val = 1; // 1 - error, 0 - success

  // extern "C" GID_DLL_EXPORT void hello() {
  // printf( "-->hello.\n");
  // printf( "-->hello: %d\n", hola);
  printf( "importing %s: %s\n", filename, res ? "Ok" : "can not be read");
  if ( res) {
    // ret_val = create_demo_triangs();
    ret_val = obj_2_gid( filename);
  } else {
    char buf[ 8192];
    sprintf( buf, "Problems reading the Object '%s'.", filename);
    set_err_str( buf);
    ret_val = 1;
  }
  return ret_val;
}

extern "C" GID_DLL_EXPORT const char *GiD_PostImportGetLibName( void) {
  return "Wavefront Objects import"; 
}

extern "C" GID_DLL_EXPORT const char *GiD_PostImportGetFileExtensions( void) {
  return "{{Wavefront Objects} {.obj}} {{All files} {*}}";
}

extern "C" GID_DLL_EXPORT const char *GiD_PostImportGetDescription( void) {
  return "Wavefront OBJ import plugin for GiD"; 
}

extern "C" GID_DLL_EXPORT const char *GiD_PostImportGetErrorStr( void) {
  return _G_err_str; 
}
