#Amelet read/write plugin

if { [GiDVersionCmp 10.1.0d] < 0 } {
    #this plugin require GiD 10.1.0d or higher
    WarnWinText [_ "This plugin requires GiD %s or later" 10.1.0d]
}

namespace eval Amelet {
    variable version 1.3 ;#Amelet plugin version
    variable compress_level 9
}

#AMELET_bar2=1,AMELET_bar3=2,AMELET_tri3=11,AMELET_tri6=12,
#AMELET_quad4=13,AMELET_quad8=14,AMELET_tetra4=101,AMELET_pyra5=102,AMELET_penta6=103,AMELET_hexa8=104,
#AMELET_plane=15,AMELET_circle=16,AMELET_cylinder=105,AMELET_cone=106,AMELET_sphere=107
proc Amelet::ElementCode { elemtype nnodes } {
    if { $elemtype == "Linear" } {        
	if { $nnodes == 2 } {
	    return 1
	} else {
	    return 2
	}
    } elseif { $elemtype == "Triangle" } {
	if { $nnodes == 3 } {
	    return 11
	} else {
	    return 12
	}
    } elseif { $elemtype == "Quadrilateral" } {
	if { $nnodes == 4 } {
	    return 13
	} else {
	    return 14
	}
    } elseif { $elemtype == "Tetrahedra" } {
	return 101
    } elseif { $elemtype == "Pyramid" } {
	return 102
    } elseif { $elemtype == "Prism" } {
	return 106
    } elseif { $elemtype == "Hexahedra" } {
	return 104
    } elseif { $elemtype == "Circle" } {
	return 16
    } elseif { $elemtype == "Sphere" } {
	return 107
    }
    return 0
}

proc Amelet::ElementEntityType { elemtype } {
    if { $elemtype == "Tetrahedra" || $elemtype == "Hexahedra" || $elemtype == "Pyramid"
	 || $elemtype == "Prism" || $elemtype == "Sphere"} {
	return volume
    } elseif { $elemtype == "Triangle" || $elemtype == "Quadrilateral" || $elemtype == "Circle" } {
	return surface
    } elseif { $elemtype == "Linear" } {        
	return edge
    } else {
	return unknown
    }
}

proc Amelet::IsValidGroupName { name } {
    if { [string first / $name] != -1 } {
	return 0
    }
    if { [string first . $name] != -1 } {
	return 0
    }
    return [string is ascii $name]   
}

proc Amelet::IsValidAttributeName { name } {
    return [string is ascii $name]
}

#set parent "" as root
#limitations:
#-prefix name with folder_ to avoid leaves repeating name
#-Amelet names must have max 20 characters (and maybe must be ASCII?)
#-only a level of folders in gid is valid, althought Amelet allow create groupGroups of groupGroups
proc Amelet::WriteLayerUnstructuredMesh { h meshpath parent } {
    variable compress_level
    variable NewElementNum
    
    set LayerNames [GetChildLayerNames $parent]
    foreach item $LayerNames {
	if { $parent != "" } {
	    set layername $parent$item
	} else {
	    set layername $item
	}
	if { [IsFolderOfLayers $item] } {
	    #only implemented one level (but it is possible to create groupGroups of groupGroups)
	    set childs [list]
	    foreach child [GetChildLayerNames $layername] {
		 #lappend childs $layername$child
		 lappend childs $child
	    }
	    set groupname [string range $item 0 end-2]
	    $h set -vtype string -compress $compress_level ${meshpath}/groupGroup/$groupname $childs
	    Amelet::WriteLayerUnstructuredMesh $h $meshpath $layername
	} else {
	    set ids [GiD_Info layer -entities elements $layername]
	    if { $ids != "" } {            
		set entityType [Amelet::ElementEntityType [lindex [GiD_Mesh get element [lindex $ids 0]] 1]]
		set elements [list]
		foreach id $ids {
		    lappend elements $NewElementNum($id)
		}
		#set groupname $item
		set groupname [string map {// _} $layername]           
		if { ![Amelet::IsValidGroupName $groupname] } {                   
		    set groupname [base64::encode $groupname]
		}
		$h set -vtype int -compress $compress_level ${meshpath}/group/$groupname $elements                      
		$h set_attribute -type string ${meshpath}/group/$groupname type element entityType $entityType  
	    }   
	}
    }    
}


proc Amelet::WriteUnstructuredMesh { filename } {
    variable compress_level
    variable NewElementNum
    package require hdf5 1.2
    
    array unset NewElementNum

    set h hdf1
    if { [file exists $filename] } {
	if { [catch { file delete $filename } err] } {
	    WarnWin $err
	    return 1
	}
    }
    hdf5 $h $filename
    
    $h set_attribute -type string / FORMAT AMELETHDF AMELETHDF_FORMAT_VERSION 1.0.0

    set meshgroup /mesh/gid
    set meshpath $meshgroup/all   
    $h create_group /mesh $meshgroup $meshpath 
    $h set_attribute -type string $meshpath type unstructured
    $h create_group $meshpath/group $meshpath/groupGroup $meshpath/selectorOnMesh
	
    set nodes [GiD_Info mesh nodes -array]
    lassign $nodes ids coordinates
    lassign $coordinates xs ys zs
    set data [list]
    foreach x $xs y $ys z $zs {
	lappend data $x $y $z
    }
    $h set -vtype float -compress $compress_level -ncolumns 3 ${meshpath}/nodes $data
    
    #Amelet nodes must stored with implicit number 0,..,n-1    
    set inode -1
    foreach id $ids {
	set NewNodeNum($id) [incr inode]
    }    

    set elementTypes [list]
    set elementNodes [list]
    set elements [GiD_Info mesh elements Any -array]
    set ielement -1
    foreach item $elements {
	lassign $item elemtype ids connectivities materials
	#test only for triangles 
	set nnodes [llength $connectivities]
	set nodes [list]
	if { $elemtype == "Linear" } {  
	    if { $nnodes == 2 } {
		lassign $connectivities n1s n2s
		foreach n1 $n1s n2 $n2s {              
		    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2)
		}
	    } elseif { $nnodes == 3 } {
		lassign $connectivities n1s n2s n3s
		foreach n1 $n1s n2 $n2s n3 $n3s {              
		    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3)
		}
	    }
	} elseif { $elemtype == "Triangle" } {                         
	    if { $nnodes == 3 } {
		lassign $connectivities n1s n2s n3s
		foreach n1 $n1s n2 $n2s n3 $n3s {              
		    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3)
		}
	    } elseif { $nnodes == 6 } {
		lassign $connectivities n1s n2s n3s n4s n5s n6s
		foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s {              
		    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) \
		        $NewNodeNum($n4) $NewNodeNum($n5) $NewNodeNum($n6)
		}
	    }
	} elseif { $elemtype == "Quadrilateral" } {
	    if { $nnodes == 4 } {
		lassign $connectivities n1s n2s n3s n4s
		foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s {
		    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4)
		}
	    } elseif { $nnodes == 8 || $nnodes == 9 } {
		if { nnodes == 8 } {
		    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s
		} else {
		    #9 nodes ignore the last center node
		    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s
		}
		foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s n7 $n7s n8 $n8s {
		    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) \
		        $NewNodeNum($n5) $NewNodeNum($n6) $NewNodeNum($n7) $NewNodeNum($n8)                        
		}
	    }
	} elseif { $elemtype == "Tetrahedra" } {
	    if { $nnodes == 4 } {
		lassign $connectivities n1s n2s n3s n4s
	    } else {                
		#10 nodes ignore all quadratic ones                
		lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s
	    }
	    foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s {
		lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4)
	    }
	} elseif { $elemtype == "Pyramid" } {
	    if { $nnodes == 5 } {
		lassign $connectivities n1s n2s n3s n4s n5s
	    } else {                
		#13 nodes ignore all quadratic ones
		lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s n11s n12s n13s 
	    }
	    foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s {
		lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) $NewNodeNum($n5)
	    }
	} elseif { $elemtype == "Prism" } {
	    if { $nnodes == 6 } {
		lassign $connectivities n1s n2s n3s n4s n5s n6s
	    } else {                
		#15 nodes ignore all quadratic ones
		lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s n11s n12s n13s n14s n15s
	    }
	    foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s {
		lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) \
		    $NewNodeNum($n4) $NewNodeNum($n5) $NewNodeNum($n6)
	    }
	} elseif { $elemtype == "Hexahedra" } {
	    if { $nnodes == 8 } {
		lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s
	    } elseif { $nnodes == 20 } {
		#20 nodes ignore all quadratic ones                
		lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s \
		    n11s n12s n13s n14s n15s n16s n17s n18s n19s n20s
	    } else {                
		#27 nodes ignore all quadratic ones
		lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s \
		    n11s n12s n13s n14s n15s n16s n17s n18s n19s n20s \
		    n21s n22s n23s n24s n25s n26s n27s
	    }
	    foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s n7 $n7s n8 $n8s {
		lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) \
		    $NewNodeNum($n5) $NewNodeNum($n6) $NewNodeNum($n7) $NewNodeNum($n8)
	    }
	} elseif { $elemtype == "Circle" } {
	    continue
	} elseif { $elemtype == "Sphere" } {
	    continue
	} elseif { $elemtype == "Point" } {
	    continue
	} else {
	    continue
	}
	set elementTypes [concat $elementTypes [lrepeat [llength $ids] [Amelet::ElementCode $elemtype $nnodes]]]
	set elementNodes [concat $elementNodes $nodes]
	#Amelet elements must stored with implicit number 0,..,n-1
	foreach id $ids {
	    set NewElementNum($id) [incr ielement]
	}
    }
    $h set -vtype char -compress $compress_level ${meshpath}/elementTypes $elementTypes
    $h set -vtype int -compress $compress_level ${meshpath}/elementNodes $elementNodes    

    Amelet::WriteLayerUnstructuredMesh $h $meshpath ""
   
    $h close    
    return 0
}

#Amelet::WriteUnstructuredMesh {C:\gid project\gid\hdf5tcl\examples\test_3.h5}

#num starting from 1
#i,j,k starting from 0
proc Amelet::GetNumFromIjk { n0 n01 i j k } {
    return [expr {$k*$n01+$j*$n0+$i+1}]
}

#n0 num grid points in i direction
#n01=n0*n1 num grid points by ij plane
#num starting from 1
#i,j,k starting from 1 !!
proc Amelet::GetIjkFromNum { n0 n01 num } {
    #int num=k*(ngridpoints[0]*ngridpoints[1])+j*ngridpoints[0]+i+1;      
    incr num -1
    set k [expr $num/$n01+1]
    set rest [expr $num%$n01]
    set j [expr $rest/$n0+1]
    set i [expr $rest%$n0+1]
    return "$i $j $k"
}


#set parent "" as root
#limitations:
#-prefix name with folder_ to avoid leaves repeating name
#-Amelet names must have max 20 characters (and maybe must be ASCII?)
#-only a level of folders in gid is valid, althought Amelet allow create groupGroups of groupGroups
proc Amelet::WriteLayerStructuredMesh { h meshpath n0 n01 entityType parent } {
    variable compress_level
    
    set LayerNames [GetChildLayerNames $parent]
    foreach item $LayerNames {
	if { $parent != "" } {
	    set layername $parent$item
	} else {
	    set layername $item
	}
	if { [IsFolderOfLayers $item] } {
	    #only implemented one level (but it is possible to create groupGroups of groupGroups)
	    set childs [list]
	    foreach child [GetChildLayerNames $layername] {
		 #lappend childs $layername$child
		 lappend childs $child
	    }
	    set groupname [string range $item 0 end-2]
	    $h set -vtype string -compress $compress_level ${meshpath}/groupGroup/$groupname $childs
	    Amelet::WriteLayerStructuredMesh $h $meshpath $n0 $n01 $entityType $layername
	} else {
	    set ids [GiD_Info layer -entities elements $layername]
	    if { $ids != "" } {
		set elements [list]                        
		foreach id $ids {                                
		    set idnodes [lrange [GiD_Mesh get element $id] 3 end]                    
		    set ijk [GetIjkFromNum $n0 $n01 [lindex $idnodes 0]] ;#assumed first node as lower left corner!!
		    lassign $ijk i j k
		    lappend elements $i $j $k [expr $i+1] [expr $j+1] [expr $k+1]
		}
		#set groupname $item
		set groupname [string map {// _} $layername]                
		if { ![Amelet::IsValidGroupName $groupname] } {
		    package require base64
		    set groupname [base64::encode $groupname]
		}
		$h set -vtype int -compress $compress_level -ncolumns 6 ${meshpath}/group/$groupname $elements           
		$h set_attribute -type string ${meshpath}/group/$groupname type element entityType $entityType
	    }
	}
    }    
}

proc Amelet::WriteStructuredMesh { filename } {
    variable compress_level
    package require hdf5 1.2

    set h hdf1
    if { [file exists $filename] } {
	if { [catch { file delete $filename } err] } {
	    WarnWin $err
	    return 1
	}
    }
    hdf5 $h $filename
    
    $h set_attribute -type string / FORMAT AMELETHDF AMELETHDF_FORMAT_VERSION 1.0.0

    set meshgroup /mesh/gid
    set meshpath $meshgroup/all   
    $h create_group /mesh $meshgroup $meshpath 
    $h set_attribute -type string $meshpath type structured
    $h create_group $meshpath/cartesianGrid $meshpath/group $meshpath/groupGroup $meshpath/normal $meshpath/selectorOnMesh
	
    set pmin [GiD_Cartesian get corner]
    set boxsize [GiD_Cartesian get boxsize]
    set ngridpoints [GiD_Cartesian get ngridpoints]
    set dimension [GiD_Cartesian get dimension]

    lassign [GiD_Cartesian get coordinates] c(x) c(y) c(z)

    foreach i {0 1 2} axe {x y z} {
	if { $i>=$dimension } break
	set n [lindex $ngridpoints $i]
	if { $c($axe) == "" } {        
	    set v [lindex $pmin $i]
	    set dv [expr {[lindex $boxsize $i]/($n-1)}]
	    set data [list]
	    for {set j 0} {$j<$n} {incr j} {
		lappend data $v
		set v [expr $v+$dv]
	    }
	} else {
	    set data $c($axe)
	}
	$h set -vtype float -compress $compress_level -ncolumns 1 ${meshpath}/cartesianGrid/$axe $data
    }
    
    set n0 [lindex $ngridpoints 0]
    set n01 [expr {$n0*[lindex $ngridpoints 1]}]
    if { $dimension ==3 } {
	set entityType volume
    } elseif { $dimension ==2 } {
	set entityType surface
    } elseif { $dimension ==1 } {
	set entityType edge
    } else {
	set entityType volume
    }


    Amelet::WriteLayerStructuredMesh $h $meshpath $n0 $n01 $entityType ""
    
    $h close    
    return 0
}


proc Amelet::Read { filename } {     
    GiD_Process Mescape Files HDF5Read $filename escape
}

proc Amelet::ReadW { } {
    set filename [Browser-ramR file read .gid [= "Read Amelet file"] \
	    {} {{{Amelet} {.h5 }} {{All files} {.*}}} ]
    if {$filename == ""} {
	return
    }
    Amelet::Read $filename
}

proc Amelet::Write { filename } {
    if { [GiD_Cartesian get iscartesian] } {	
	Amelet::WriteStructuredMesh $filename
    } else {
	Amelet::WriteUnstructuredMesh $filename	
    }
}

proc Amelet::WriteW { } {
    set filename [Browser-ramR file write .gid [= "Write Amelet file"] \
	    {} {{{Amelet} {.h5 }} {{All files} {.*}}} ]
    if {$filename == ""} {
	return
    }
    GidUtils::WaitState .gid
    Amelet::Write $filename
    GidUtils::EndWaitState .gid
    GidUtils::SetWarnLine [_ "File %s written." $filename]
}

proc Amelet::AddToMenu { } {
    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
    if { $position == -1 } {
	set position end
    }
    GiDMenu::InsertOption Files [= "Import Amelet"]... $position PRE Amelet::ReadW "" "" insertafter _
    #set position [GiDMenu::GetOptionIndex Files [list Export "GiD mesh"] PRE]
    set position [GiDMenu::GetOptionIndex Files [list Export [_ "%s mesh#C#menu" $::GidPriv(ProgName)]] PRE]    
    if { $position == -1 } {
	set position end
    }
    GiDMenu::InsertOption Files [= "Export Amelet"]... $position PRE Amelet::WriteW "" "" insertafter _
}
 
#invoke this menus changes
Amelet::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc Amelet::AddToMenu
 

#register the proc to be automatically called when dropping a file
GiD_RegisterExtensionProc ".h5" PRE Amelet::Read