
#Prog: Will we ALWAYS find the node definition before the elements??    
#Prog: Will we ALWAYS finish each block with a blank line?? 
#Prog: Referr everything to the origin that is written in the input file -> Enter the coordinate system with GiD Tools

#Prog: We are loosing the information of the origin (NOTE: there are to different defitions for the
#local coordinates: Spherical and Cylindrical, with different tags: XorR,Y,Z and XorR,T,Z)
#Prog: Maybe we can check that all the coordinates are OK before writing (2D version for instance)  

namespace eval SAP2000 {
}
   
proc InitGIDProject { dir } {
    GiDMenu::InsertOption "Files#C#menu" [list "Import#C#menu" [= "SAP2000 mesh#C#menu"]...] 9 PRE \
	SAP2000::readwindow "" "" "insert"   
    GiDMenu::UpdateMenus
}

proc SAP2000::readwindow { } {    
    #User introduce the path of a SAP2000 file    
    set types [list [list [= "SAP2000 file"] ".s2k"]]     
    set filename [Browser-ramR file read .gid [= "SAP2000 file read"] "" $types]    
    if { $filename != "" } {
	SAP2000::readfile $filename
	GidUtils::SetWarnLine [= "File read"]
	GiD_Process MEscape Meshing MeshView
    }
    return 1
}

proc SAP2000::readfile { filename } {   
    #General structure of the reading process    
    if { [catch { set fileid [open $filename r] }] } {
	return 0
    }
    while { ![eof $fileid] } {
	set read_line [string trim [gets $fileid]]
	if { $read_line == "END TABLE DATA" } {
	    break
	}
	if { [catch { set item [lindex $read_line 0] } ] } { continue }
	switch -- $item {
	    TABLE: {
		if { [catch { set table [lindex $read_line 1] } ] } { continue }                
		if {$table == "JOINT COORDINATES"} {                 
		   SAP2000::read_joint_coordinates $fileid  
		} elseif {$table == "CONNECTIVITY - FRAME"} {            
		    SAP2000::read_connectivity_linear $fileid Frame
		} elseif {$table == "CONNECTIVITY - CABLE"} {            
		    SAP2000::read_connectivity_linear $fileid Cable
		} elseif {$table == "CONNECTIVITY - LINK"} {            
		    SAP2000::read_connectivity_linear $fileid Link
		} elseif {$table == "CONNECTIVITY - TENDON"} {            
		    SAP2000::read_connectivity_linear $fileid Tendon
		} elseif {$table == "CONNECTIVITY - AREA"} {          
		    SAP2000::read_connectivity_area $fileid   
		} elseif {$table == "CONNECTIVITY - SOLID"} {                 
		    SAP2000::read_connectivity_solid $fileid   
		} else {
		    #...
		}           
	    }
	}
    }    
    close $fileid    
    return 1    
}

proc SAP2000::read_joint_coordinates { fileid } {        
    #read_line sample:
    #Joint=1   CoordSys=GLOBAL   CoordType=Cylindrical   XorR=864   T=90   Z=0   SpecialJt=No   GlobalX=5.29029944851267E-14   GlobalY=864   GlobalZ=0
    while { ![eof $fileid] } {   
	set read_line [string trim [gets $fileid]]
	if { $read_line == "" } {
	    break
	}        
	array set V [string map "= { }" $read_line]
		              
	GiD_Mesh create node $V(Joint) "$V(GlobalX) $V(GlobalY) $V(GlobalZ)"                 
    }   
    return 1    
}

#type could be Frame, Cable
proc SAP2000::read_connectivity_linear { fileid type} {
    #read_line sample:

    while { ![eof $fileid] } {   
	set read_line [string trim [gets $fileid]]
	if { $read_line == "" } {
	    break
	}
	array set V [string map "= { }" $read_line]
	GiD_Mesh create element $V($type) linear 2 "$V(JointI) $V(JointJ)"
    }
    return 1;
}

proc SAP2000::read_connectivity_area { fileid } {
    #read_line sample:

    while { ![eof $fileid] } {   
	set read_line [string trim [gets $fileid]]
	if { $read_line == "" } {
	    break
	}        
	array set V [string map "= { }" $read_line]
	if { $V(NumJoints) == 3 } {
	    GiD_Mesh create element $V(Area) triangle 3 "$V(Joint1) $V(Joint2) $V(Joint3)"
	} elseif { $V(NumJoints) == 4 } {
	    GiD_Mesh create element $V(Area) quadrilateral 4 "$V(Joint1) $V(Joint2) $V(Joint4) $V(Joint3)"
	} else {
	}

    }
    return 1;        
}


proc SAP2000::read_connectivity_solid { fileid } {    
    #read_line sample:
    #Solid=1   Joint1=1   Joint2=2   Joint3=3   Joint4=4   Joint5=5   Joint6=6   Joint7=7   Joint8=8   Volume=9557820.93994232   CentroidX=311.161267687229   CentroidY=751.209752535714   CentroidZ=240
    while { ![eof $fileid] } {   
	set read_line [string trim [gets $fileid]]
	if { $read_line == "" } {
	    break
	}        
	array set V [string map "= { }" $read_line]
	GiD_Mesh create element $V(Solid) hexahedra 8 \
	    "$V(Joint1) $V(Joint2) $V(Joint4) $V(Joint3) $V(Joint5) $V(Joint6) $V(Joint8) $V(Joint7)"
    }       
    return 1;
}