
proc doRestartGiD { } {
    set args $::argv

    if { "$::GidProv(restartCurrentProject)" != "UNNAMED"} {
	set args $::GidProv(restartCurrentProject)
    }

    # saving the preferences:
    set filename [ GiveGidDefaultsFile]
    writedefaults $filename;#kernel values
    WriteTCLDefaultsInFile $filename ;#tcl values
    
    # WarnWin "argv0 = $::argv0"
    if { [ info exists ::env(INI_ARG_0)]} {
	set exec_prog $::env(INI_ARG_0)
    } else {
	#set exec_prog [ file rootname $::argv0]
	set exec_prog $::argv0
    }
    # WarnWin "$exec_prog 'argv: $::argv'"
    # WarnWin "exec-$exec_prog-$args-"
    destroy .gid
    update
    update idletasks
    set err [ catch {
	# exec $exec_prog {*}$::argv
	if { [ llength $::argv] == 0} {
	    exec $exec_prog
	} else {
	    exec $exec_prog "$args"
	}
    } errTxt ]
    # if { $err} {
    #         WarnWin $errTxt
    # }
    exit
}

proc checkMaxAttemptsRestart { } {
    if { $::GidPriv(restartMaxAttempts) <= 0} {
	# max number of attempts, now aborting restart.
	WarnWinTk [_ "%s could not be restarted automatically.\nPlease,\
 as soon as you can quit the program\nand\
 start it again so that the new settings\ncan\
 be applied." $::GidPriv(ProgName)]
	return 0
    } else {
	return 1
    }
}

proc checkChangesAndRestart { } {
    if { ![ checkMaxAttemptsRestart]} {
	return
    }
    set notSaved [ GiD_Info Project AreChanges]
    if { $notSaved} {
	after 1000 checkChangesAndRestart
    } else {
	doRestartGiD
    }
}

proc checkIfInPreAndRestart {} {
    if { ![ checkMaxAttemptsRestart]} {
	return
    }
    set modo [ GiD_Info Project ViewMode]
    if { ( $modo eq "POSTUSE") || ( $modo eq "GRAPHUSE")} {
	incr ::GidPriv(restartMaxAttempts) -1
	after 1000 checkIfInPreAndRestart
    } else {
	# ask the user to save any modifications:
	GiD_Process escape escape escape escape Files New
	after 1000 checkChangesAndRestart
    }
}

proc RestartGiD { { w .gid}} {
    set resp [ tk_messageBox -parent $w -default no -title [_ "%s restart" $::GidPriv(ProgName)] \
		   -icon question -type yesno \
		   -message [_ "In order to apply the new settings %s needs to be reinitiated.\nRestart %s now?" \
		                 $::GidPriv(ProgName)  $::GidPriv(ProgName)]
	       ]
    if { "$resp" == "no"} {
	return
    }

    # if we are in Post mode, change to pre
    set modo [GiD_Info Project ViewMode]
    if { ( $modo eq "POSTUSE") || ( $modo eq "GRAPHUSE")} {
	GiD_Process escape escape escape escape Preprocess
    }

    # wait at max. 15 seconds to solve for all issues: in post, save changes, ....
    # if not, abort restart
    set ::GidPriv(restartMaxAttempts) 15

    set ::GidProv(restartCurrentProject) [ lindex [ GiD_Info project] 1]

    checkIfInPreAndRestart
}

proc processBgColor {color fm} {

    set colorStr " "

    if {$fm == "gidToTcl"} {
	# Format RRR#GGG#BBB to #2x2x2x
	scan [string range $color 0 end] %3i%c%3i%c%3i col1 sp1 col2 sp2 col3
	set colorStr [format %c%02x%02x%02x $sp1 $col1 $col2 $col3]

    } elseif {$fm == "tclToGid"} {
	# Format #2x2x2x to RRR#GGG#BBB
	puts $color
	scan [string range $color 0 end] %c%2x%2x%2x sp1 col1 col2 col3
	
	puts $col1
	puts $col2
	puts $col3
	set colorStr [format %03i%c%03i%c%03i $col1 $sp1 $col2 $sp1 $col3]
    }

    return $colorStr
}


proc DefineFuenteSiNoEsta { nombre_fnt} {
    if { [ lsearch [ font names] $nombre_fnt] == -1 } {
	#default: -adobe-helvetica-medium-r-normal--12-120-75-75-p-67-iso8859-1
	#Helvetica 12 normal roman
	font create $nombre_fnt
	set com_font [GiD_Set $nombre_fnt]
	# WarnWinText "$com_font"
	# WarnWinText "font conf CommentsFont [ font actual $com_font]"
	eval [ concat font conf $nombre_fnt [ font actual $com_font]]
	# ajustamos tamao
	if { [ regexp {^-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*-([0-9]+)-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*-[^-]*$} $com_font dum fnt_size]} {
	    font conf $nombre_fnt -size [ font metrics $com_font -ascent]
	}
    }
}

proc AcceptGiDFont { nombre_fnt} {
    global ChangeVarInfo GidPriv tcl_platform GIDDEFAULTTCL

    if { $tcl_platform(platform) != "windows" } {
	set wght [ font actual $nombre_fnt -weight]
	if { "$wght" == "normal"} { set wght medium}
	set slant [ string index [ font actual $nombre_fnt -slant] 0]
	set fnt_family [ font actual $nombre_fnt -family]
	#set fnt_size [ font actual $nombre_fnt -size]
	set fnt_size [ font metrics $nombre_fnt -ascent]
	set com_fnt "-*-$fnt_family-$wght-$slant-*-*-$fnt_size-*-*-*-*-*-*-*"
    } else {
	set com_fnt ""
	lappend com_fnt [ font actual $nombre_fnt -family]
	lappend com_fnt [ font actual $nombre_fnt -size]
	lappend com_fnt [ font actual $nombre_fnt -weight]
	lappend com_fnt [ font actual $nombre_fnt -slant]
    }

    GiD_Set $nombre_fnt $com_fnt
}

proc AcceptAsianFont { nombre_fnt} {
    # WarnWinText "nombre_fnt = $nombre_fnt"
    # WarnWinText "::ChangeVarInfo(AsianFonts) = $::ChangeVarInfo(AsianFonts)"
    # WarnWinText "::ChangeVarInfo(AsianFont,family) = $::ChangeVarInfo(AsianFont,family)"
    # WarnWinText "::ChangeVarInfo(AsianFont,size) = $::ChangeVarInfo(AsianFont,size)"
    foreach ifnt $::ChangeVarInfo(AsianFonts) {
	set idioma [ lindex $ifnt 0]
	set nombre [ lindex $ifnt 1]
	#set estilo [ lindex $ifnt 2]
	#set tamanno_ant [ lindex $ifnt 3]
	#set txt "${idioma}: $nombre $estilo"
	set txt "${idioma}: $nombre"
	if { [ string equal $txt $nombre_fnt]} {
	    #WarnWinText "[ llength $ifnt] = $ifnt"
	    #GiD_Set AsianFont [ lreplace $ifnt 3 3 $::ChangeVarInfo(AsianFont,size)]
	    GiD_Set AsianFont $ifnt
	    break
	}
    }
}

proc AcceptPGFGiDFont { nombre_fnt} {
    #set idx $::ChangeVarInfo($nombre_fnt,texto)
    #set ident $::PGFFonts($idx)
    #set tam $::ChangeVarInfo($nombre_fnt,tamanno)
    #set ifnt "$ident $tam"

    set bPGFFonts 1
    if { [ info commans GiD_PGF_Fonts] != ""} {
	#set lst_ttf_fonts [ GiD_PGF_Fonts get]
	set bPGFFonts 1
    } else {
	# set lst_ttf_fonts {}
	set bPGFFonts 0
    }
    if { !$bPGFFonts} { return}
    set err [
	     catch {
		 set ifnt "$::PGFFonts($::ChangeVarInfo($nombre_fnt,texto)) $::ChangeVarInfo($nombre_fnt,tamanno)"
		 GiD_Set $nombre_fnt $ifnt
	     }]
    if { $err} {
	WarnWinTk [_ "Problems appeared when trying to set the new fonts.\nIf the language of the messages have been changed,\n it is recomended to restart the program."]
    }
    # set fuente_sel $::ChangeVarInfo($nombre_fnt,texto)
    # set idx [ lsearch $::PGFListaFuentes $fuente_sel]
    # if { $idx != -1} {
    #         set info_fuente [ lindex [ lindex $::PGFInfoFuentes $idx] 0]
    #         GiD_Set $nombre_fnt $info_fuente $::ChangeVarInfo($nombre_fnt,tamanno)
    # } else {
    #         WarnWinText "Fuente no encontrada: '$fuente_sel'"
    #         # WarnWinText "Lista fuentes:"
    #         # foreach if $::PGFListaFuentes {
    #         #     WarnWinText "\t$if"
    #         # }
    # }
}

proc AcceptPGFModoRender { } {
    # GiD_Set PGF_ModoRender PGF_$::ChangeVarInfo(PGF_ModoRender,texto)
    GiD_Set PGF_ModoRender $::ChangeVarInfo(PGF_ModoRender,texto)
}

#change from GiD color varible style rrr#ggg#bbb (xxx from 0 to 255)
#to Tk style #rrggbb hexadecimals
proc GiDColorToTkColor { colorgid } {
    scan $colorgid  %03d#%03d#%03d r g b
    return #[format %02x%02x%02x $r $g $b]
}

#change from Tk style color #rrggbb hexadecimals
#to GiD color varible style rrr#ggg#bbb (xxx from 0 to 255)
proc TkColorToGiDColor { tkcolor } {
    scan $tkcolor #%2x%2x%2x r g b
    return [format %03d#%03d#%03d $r $g $b]
}


proc FillPrefValuesFromGid {} {
    global ChangeVarInfo GidPriv

    if { ![winfo exists .gid.wvar] } return

    if { ![ info exists ChangeVarInfo(type)]} return

    if { $ChangeVarInfo(type) == "GENERAL" } {

	set ChangeVarInfo(Dialogbrowser) [GiD_Set DialogWrowser]

	set ChangeVarInfo(SplashWindow) [GiD_Set SplashWindow]

	set input [GiD_Set CreateAlwaysNewPoint]
	if { $input == 1 } {
	    set ChangeVarInfo(Createnewpoint) Always
	} elseif { $input == 0 } {
	    set ChangeVarInfo(Createnewpoint) Ask
	} elseif { $input == -2 } {
	    set ChangeVarInfo(Createnewpoint) Never
	} else {set ChangeVarInfo(Createnewpoint) No }

	set ChangeVarInfo(Displaycoordinates) \
		[GiD_Set DisplayCoordinates]

	set input [GiD_Set AutomaticRedraw]
	if { $input == 0 } {
	    set ChangeVarInfo(AutomaticRedraw) "None"
	} elseif { $input == 1 } {
	    set ChangeVarInfo(AutomaticRedraw) "All windows"
	} elseif { $input == 2 } {
	    set ChangeVarInfo(AutomaticRedraw) "Current window"
	} else {
	    set ChangeVarInfo(AutomaticRedraw) "Current window"
	}
	
	set input [GiD_Set UseMoreWindows]
	if { $input == 1 } {
	    set ChangeVarInfo(Usemorewindows) normal
	} elseif { $input == 2 } {
	    set ChangeVarInfo(Usemorewindows) beginner
	} else {
	    #set ChangeVarInfo(Usemorewindows) no
	    set ChangeVarInfo(Usemorewindows) normal
	}

	set ChangeVarInfo(MaxNumOfRecentProjects) [GiD_Set MaxNumOfRecentProjects]
	set ChangeVarInfo(MaxNumOfRecentPostFiles) [GiD_Set MaxNumOfRecentPostFiles]
	set ChangeVarInfo(MaxNumOfRecentViewFiles) [GiD_Set MaxNumOfRecentViewFiles]
	set ChangeVarInfo(ReloadLastModel) [GiD_Set ReloadLastModel]

	set ChangeVarInfo(LoadMeshingPrefOfModel) [GiD_Set LoadMeshingPrefOfModel]
	
	set ChangeVarInfo(GID_OMP_NUM_THREADS) [GiD_Set GiD_OMP_NUM_THREADS]        
	
	set ChangeVarInfo(Sound) [GiD_Set Sound]
	set ChangeVarInfo(SmallWinSelecting) [GiD_Set SmallWinSelecting]

	set input [GiD_Set BatchFileToWrite]
	if { $input == "UNKNOWN" } {
	    set ChangeVarInfo(WriteBatchCheck) 0
	    set ChangeVarInfo(WriteBatchFile) ""
	} else {
	    set ChangeVarInfo(WriteBatchCheck) 1
	    set ChangeVarInfo(WriteBatchFile) $input
	}
	set ChangeVarInfo(WriteRotationsInBatch) \
		[GiD_Set WriteRotationsInBatch]
	
	set input [GiD_Set BackUpMinutes]
	set ChangeVarInfo(BackupTime) $input
	if { $input == 0 } {
	    set ChangeVarInfo(BackupCheck) 0
	} else {
	    set ChangeVarInfo(BackupCheck) 1
	}

	set ChangeVarInfo(Language) [::GidUtils::GetCurrentLanguage]

    } elseif { $ChangeVarInfo(type) == "GRAPH" } {

	set ChangeVarInfo(SoftwareOpenGL) $GidPriv(SoftwareOpenGL)
	set ChangeVarInfo(OGL_emulateFrontBuffer) [ GiD_Set OGL_emulateFrontBuffer]

	set ChangeVarInfo(Lightsmoothedelements) \
		[GiD_Set LightSmoothedElems]

	set ChangeVarInfo(AngleSmoothed) [format %g \
		[expr 57.295779513082323*acos([GiD_Set \
		CosSmoothedElems])]]

	set input [GiD_Set DrawSurfaceMode]
	switch $input {
	    0 { set ChangeVarInfo(DrawSurfaceMode) "None" }
	    1 { set ChangeVarInfo(DrawSurfaceMode) "Boundary lines" }
	    2 { set ChangeVarInfo(DrawSurfaceMode) "Isoparametric lines" }
	    3 { set ChangeVarInfo(DrawSurfaceMode) "Both modes" }
	}

	set ChangeVarInfo(DrawQuadratic) [GiD_Set DrawQuadraticElements]

	set ChangeVarInfo(LabelOptions) [GiD_Set LabelOptions]
	
	set ChangeVarInfo(CurvePrecision) \
		[GiD_Set CurvePrecision]

	set ChangeVarInfo(SurfacePrecision) \
		[GiD_Set SurfacePrecision]

	foreach i {BackgroundColor ColorPoints ColorLines ColorPolyLines \
		ColorSurfaces ColorSurfsIsoparametric ColorVolumes \
		ColorNodes ColorElements} {
	    set ChangeVarInfo($i) [GiDColorToTkColor [GiD_Set $i]]
	}

	set ChangeVarInfo(GUIAppearance) $GidPriv(GUIAppearance)
	set ChangeVarInfo(BackColorType) [ GiD_Set BackColorType]
	ChangeColorEnableDisableWidgets
	set ChangeVarInfo(BackColorTop) [ GiD_Set BackColorTop]
	set ChangeVarInfo(BackColorBottom) [ GiD_Set BackColorBottom]
	ChangeColorFondoGiD BackColorTop 1
	ChangeColorFondoGiD BackColorBottom 1
	set ChangeVarInfo(BackColorSense) [ GiD_Set BackColorSense]
	ChangeColorSenseFondoGiD $ChangeVarInfo(BackColorSense)
	
	set ChangeVarInfo(FastRotation) \
		[GiD_Set FastRotation(Active)]
	if { $ChangeVarInfo(FastRotation) == "" } { set ChangeVarInfo(FastRotation) 0 }

	set ChangeVarInfo(AutomaticRotationCenter) \
	    [ GiD_Set AutomaticRotationCenter]
	if { $ChangeVarInfo(AutomaticRotationCenter) == ""} { set ChangeVarInfo(AutomaticRotationCenter) 1}

	set ChangeVarInfo(FRPoints) \
		[GiD_Set FastRotation(Points)]
	if { $ChangeVarInfo(FRPoints) == "" } { set ChangeVarInfo(FRPoints) 1 }
	
	set ChangeVarInfo(FRLines) \
		[GiD_Set FastRotation(Lines)]
	if { $ChangeVarInfo(FRLines) == "" } { set ChangeVarInfo(FRLines) 1 }
	
	set ChangeVarInfo(FRSurfaces) \
		[GiD_Set FastRotation(Surfaces)]
	if { $ChangeVarInfo(FRSurfaces) == "" } { set ChangeVarInfo(FRSurfaces) 1 }
	
	set ChangeVarInfo(FRVolumes) \
		[GiD_Set FastRotation(Volumes)]
	if { $ChangeVarInfo(FRVolumes) == "" } { set ChangeVarInfo(FRVolumes) 1 }
	set ChangeVarInfo(AlwaysGeom) \
		[GiD_Set FastRotation(UseAlwaysGeom)]
	
	set ChangeVarInfo(DrawGraphicObjects) \
		[GiD_Set FastRotation(DrawGraphicObjects)]
	
	set ChangeVarInfo(FRCurvePrecision) \
		[GiD_Set FastRotation(CurvePrecision)]
	if { $ChangeVarInfo(FRCurvePrecision) == "" } {
	    set ChangeVarInfo(FRCurvePrecision) 0
	}
	
    } elseif { $ChangeVarInfo(type) == "MESHING" } {

	set ChangeVarInfo(SizeTransitions) \
	    [GiD_Set SizeTransitionsFactor]
	
	set ChangeVarInfo(BoundaryWeightedTransition) \
	    [GiD_Set BoundaryWeightedTransition]
	
	set ChangeVarInfo(VolumeMesher) \
	    [GiD_Set VolumeMesher]

	set ChangeVarInfo(MeshUntilEnd) \
	    [GiD_Set MeshUntilEnd]
	
	set ChangeVarInfo(AvoidElementsInBoundary) \
	    [GiD_Set AvoidElementsInBoundary]
	
	set ChangeVarInfo(SymmetricalStructuredTriangles) \
	    [GiD_Set SymmetricalStructuredTriangles]
	
	set ChangeVarInfo(SymmetricalStructuredTetrahedra) \
	    [GiD_Set SymmetricalStructuredTetrahedra]
	
	set ChangeVarInfo(AllowAutomaticStructured) \
	    [GiD_Set AllowAutomaticStructured]

	set ChangeVarInfo(NoMeshFrozenLayers) \
		[GiD_Set NoMeshFrozenLayers]

	set ChangeVarInfo(AutomaticCorrectSizes) \
		[GiD_Set AutomaticCorrectSizes]

	set ChangeVarInfo(SurfaceMesher) \
		[GiD_Set SurfaceMesher]
	
	set ChangeVarInfo(QuadraticType) \
		[GiD_Set QuadraticType]        
	
	set input [GiD_Set ForceMeshEntities]
	if { $input & 1 } {
	    set ChangeVarInfo(ForceMeshPoints) 1
	} else {
	    set ChangeVarInfo(ForceMeshPoints) 0
	}
	if { $input & 2 } {
	    set ChangeVarInfo(ForceMeshLines) 1
	} else {
	    set ChangeVarInfo(ForceMeshLines) 0
	}
	if { $input & 4 } {
	    set ChangeVarInfo(ForceMeshSurfaces) 1
	} else {
	    set ChangeVarInfo(ForceMeshSurfaces) 0
	}
	
	set input [GiD_Set HighQualitySmoothing]
	if { $input == 0 } {
	    set ChangeVarInfo(HighQualitySmoothing) Normal
	} elseif { $input == 1 } {
	    set ChangeVarInfo(HighQualitySmoothing) HighAngle
	} elseif { $input == 2 } {
	    set ChangeVarInfo(HighQualitySmoothing) HighGeom
	}
	#else {set ChangeVarInfo(HighQualitySmoothing) Normal }
	set ChangeVarInfo(Rjump,TakeCareCurvatures) \
	    [GiD_Set Rjump(TakeCareCurvatures)]
	
	set ChangeVarInfo(Rjump,TakeCareLayers) \
	    [GiD_Set Rjump(TakeCareLayers)]
	
	set ChangeVarInfo(Rjump,TangencyLimitAngle) \
	    [GiD_Set Rjump(TangencyLimitAngle)]
	
	set ChangeVarInfo(Rjump,IfRjumpFailsMeshSeparately) \
	    [GiD_Set Rjump(IfRjumpFailsMeshSeparately)]
	
	set ChangeVarInfo(BoundaryLayer,GrowLaw) [GiD_Set BoundaryLayer(GrowLaw)]
	
	set ChangeVarInfo(BoundaryLayer,GrowFactor) \
	    [format %.3g [GiD_Set BoundaryLayer(GrowFactor)]]
	
	set ChangeVarInfo(BoundaryLayer,AllowPrismaticElements) \
	    [GiD_Set BoundaryLayer(AllowPrismaticElements)]
	
	set ChangeVarInfo(BoundaryLayer,MeshInSpecificLayer) \
	    [GiD_Set BoundaryLayer(MeshInSpecificLayer)]
	
	set ChangeVarInfo(ChordalError,ApplyChordalErrorToModel) \
	    [GiD_Set ChordalError(ApplyChordalErrorToModel)]

	set x [GiD_Set ChordalError(RelativeChordalError)]
	set x [format %.1f $x]
	set ChangeVarInfo(ChordalError,RelativeChordalError) $x
	
	set x [GiD_Set ChordalError(MaxChordalError)]
	set x [format %.2e $x]
	set ChangeVarInfo(ChordalError,MaxChordalError) $x
	
	set x [GiD_Set ChordalError(MinElemSize)]
	set x [format %.2e $x]
	set ChangeVarInfo(ChordalError,MinElemSize) $x
	
	
	
	set x [GiD_Set VMeshIso(AlphaShort)]
	set x [format %.1f $x]
	set ChangeVarInfo(VMeshIso,AlphaShort) $x
	
	set x [GiD_Set VMeshIso(AlphaLong)]
	set x [format %.1f $x]
	set ChangeVarInfo(VMeshIso,AlphaLong) $x
	
	
	set ChangeVarInfo(TypeOfQuadMesher) [GiD_Set TypeOfQuadMesher]
	set ChangeVarInfo(AllowQuadTriMeshes) [GiD_Set AllowQuadTriMeshes]

	set ChangeVarInfo(GID_OMP_NUM_THREADS) [GiD_Set GID_OMP_NUM_THREADS]
	
	set RballVars { Implicit MaxIter MaxIterLocal Tolerance DeltaPosition DeltaRadius \
	    MinRadius MaxRadius Preprocess Porosity Postprocess \
		Verbose Overlap Filter Search SearchType ContactFactor \
		NeighborFactor FacetFactor OverlapFactor }

	set RballFormats { %s %d %d %.3g %.3g %.3g \
		            %.3g %.3g %d %.3g %d \
		            %d %d %d %d %d %.3g \
		            %.3g %.3g %.3g }

	foreach item $RballVars fmt $RballFormats {
	    set ChangeVarInfo(Rball,$item) [format $fmt [GiD_Set Rball($item)]]
	}
	
	set CartesianVars { FromRenderMesh CondFaceElem MinSizeMainGrid WeightStart WeightEnd GridUniform }
	set CartesianFormats { %d %d %.3g %.3g %.3g %d }
	foreach item $CartesianVars fmt $CartesianFormats {
	    set ChangeVarInfo(Cartesian,$item) [format $fmt [GiD_Set Cartesian($item)]]
	}
	
    } elseif { $ChangeVarInfo(type) == "EXCHANGE" } {

	set ChangeVarInfo(AutoCollapseAfterImport) \
		[GiD_Set AutoCollapseAfterImport]

	set ChangeVarInfo(ImportTolerance) \
		[GiD_Set ImportTolerance]

	set ChangeVarInfo(CollapseIgnoringLayers) \
		[GiD_Set CollapseIgnoringLayers]

	set ChangeVarInfo(IGESIgnoreInvisible) \
		[GiD_Set IGESIgnoreInvisible]

	set ChangeVarInfo(IGESSolidsManifoldBRep) \
		[GiD_Set IGESSolidsManifoldBRep]

	set ChangeVarInfo(IGESCreateAllInLayerToUse) \
		[GiD_Set IGESCreateAllInLayerToUse]

	set ChangeVarInfo(IGESCurveOnSurfaceFrom3D) \
		[GiD_Set IGESCurveOnSurfaceFrom3D]

	set ChangeVarInfo(AutoImportTolerance) \
		[GiD_Set AutoImportTolerance]

    } elseif { $ChangeVarInfo(type) == "FONTS" } {
	GidUtils::WaitState .gid

	# las fuentes de menus, ventanas, etc.
	if { ![info exists ChangeVarInfo(fontfamilies)] } {
	    set ChangeVarInfo(fontfamilies) [lsort [font families]]
	}
	foreach i "NormalFont FixedFont BigFont" {
	    if { [ $ChangeVarInfo($i,combo) cget -values] == "" } {
		#if { [ TTKComboBoxCGet $ChangeVarInfo($i,combo) -values] == "" }
		set values ""
		foreach j $ChangeVarInfo(fontfamilies) {
		    if { $i == "FixedFont" && ![font metrics [list $j] -fixed] } {
		        continue
		    }
		    lappend values $j
		}
		$ChangeVarInfo($i,combo) configure -values $values
		#TTKComboBoxConfigure $ChangeVarInfo($i,combo) -values $values
	    }
	    foreach item {family size weight slant} {
		if { ![info exists ChangeVarInfo($i,$item)] || \
		         $ChangeVarInfo($i,$item) != [font conf $i -$item] } {
		    #if to avoid invoke traces
		    set ChangeVarInfo($i,$item) [font conf $i -$item]
		}
	    }
	}
	
	# ahora las dependientes de pre y post
	set modo [ lindex [ GiD_Info Project] 4]
	set esPost 0
	if { ( $modo eq "POSTUSE") || ( $modo eq "GRAPHUSE")} {
	    set esPost 1
	}
	if { $esPost} {

	    #GidUtils::WaitState .gid
	    set thereArePGFFonts 1
	    if { [ info commands GiD_PGF_Fonts] != ""} {
		set lst_ttf_fonts [ GiD_PGF_Fonts get]
		set thereArePGFFonts 1
	    } else {
		set lst_ttf_fonts {}
		set thereArePGFFonts 0
	    }
# WarnWin "thereArePGFFonts = $thereArePGFFonts"
	    if { [ info exists ::PGFFonts]} { unset ::PGFFonts}
	    # if { [ info exists ::PGFListaFuentes]} { unset ::PGFListaFuentes}
	    # if { [ info exists ::PGFInfoFuentes]} { unset ::PGFInfoFuentes}
	    # set ::PGFListaFuentes {}
	    # set ::PGFInfoFuentes {}
	    foreach ifont $lst_ttf_fonts {
		set txt_combo [ join [ lindex $ifont 2] ", "]
		# tabla entrada_combo --> identificador = nombre_archivo + indice
		set ::PGFFonts($txt_combo) [ lrange $ifont 0 1]
		# lappend ::PGFListaFuentes $txt_combo
		# lappend ::PGFInfoFuentes [ list [ lrange $ifont 0 1]]
	    }
	    set lst_valores_combo [ lsort [ array names ::PGFFonts]]
	    #GidUtils::EndWaitState .gid
	    # foreach ifnt $lst_valores_combo {
	    #         WarnWinText $ifnt
	    # }
	    set lst_fnts "PGF_DefaultFont PGF_AxisFont PGF_LegendFont PGF_LabelFont PGF_AsianFont PGF_TkDrawOGLFont"
	    foreach ifont $lst_fnts {
		$::ChangeVarInfo($ifont,combobox) configure -values $lst_valores_combo
		# TTKComboBoxConfigure $::ChangeVarInfo($ifont,combobox) -values $lst_valores_combo
		if { $thereArePGFFonts} {
		    set info_fnt [ GiD_Set $ifont]
		    set ::ChangeVarInfo($ifont,texto) [ join [ lindex $info_fnt 3] ", "]
		    set ::ChangeVarInfo($ifont,tamanno) [ lindex $info_fnt 2]
		    set ::ChangeVarInfo($ifont,fuente) [ lrange $info_fnt 0 1]
		} else {
		    set info_fnt {}
		    set ::ChangeVarInfo($ifont,texto) {}
		    set ::ChangeVarInfo($ifont,tamanno) {}
		    set ::ChangeVarInfo($ifont,fuente) {}
		}
	    }
	    # foreach ifnt [ $::ChangeVarInfo(PGF_DefaultFont,combobox) cget -values] {
	    #         WarnWinText $ifnt
	    # }

	    # combobox normal:
	    #$::ChangeVarInfo(PGF_ModoRender,combobox) configure -values [ list Bitmap Pixmap]
	    # es una ComboBoxGiD
	    # set tipo_render [ GiD_Set PGF_ModoRender]
	    # regsub {PGF_} $tipo_render {} ::ChangeVarInfo(PGF_ModoRender,texto)
	    # $::ChangeVarInfo(PGF_ModoRender,combobox) configure \
	    #         -keyvalues [ list PGF_Bitmap PGF_Pixmap] \
	    #         -values [ list [_ "Sharp edges"] [_ "Smooth edges"]]
	    TTKMenubuttonConfigure $::ChangeVarInfo(PGF_ModoRender,combobox) \
		-values [ list PGF_Bitmap PGF_Pixmap] \
		-labels [ list [_ "Sharp edges"] [_ "Smooth edges"]]
	    if { $thereArePGFFonts} {
		set ::ChangeVarInfo(PGF_ModoRender,texto) [ GiD_Set PGF_ModoRender]
	    } else {
		set ::ChangeVarInfo(PGF_ModoRender,texto) PGF_Bitmap
	    }
	
	} else {
	    update idletasks
	
	    #
	    foreach i "CommentsFont LabelsFont" {
		set values [GiD_Set $i]
		if { [llength $values] == 4 } {
		    foreach item {family size weight slant} value $values {
		        set ChangeVarInfo($i,$item) $value
		    }
		}
	    }
	
	    # the Comments font
	    DefineFuenteSiNoEsta CommentsFont
	    DefineFuenteSiNoEsta LabelsFont
	
	    foreach i "CommentsFont LabelsFont" {
		if { [ $ChangeVarInfo($i,combo) cget -values] == "" } {
		# if { [ TTKComboBoxCGet $ChangeVarInfo($i,combo) -values] == "" }
		    set values ""
		    foreach j $ChangeVarInfo(fontfamilies) {
		        if { $i == "FixedFont" && ![font metrics [list $j] -fixed] } {
		            continue
		        }
		        lappend values $j
		    }
		    $ChangeVarInfo($i,combo) configure -values $values
		    # TTKComboBoxConfigure $ChangeVarInfo($i,combo) -values $values
		}
		foreach item {family size weight slant} {
		    if { ![info exists ChangeVarInfo($i,$item)] || \
		             $ChangeVarInfo($i,$item) != [font conf $i -$item] } {
		        #if to avoid invoke traces
		        set ChangeVarInfo($i,$item) [font conf $i -$item]
		    }
		}
	    }
	
	    # para la fuente asiatica
	    if { [ $ChangeVarInfo(AsianFont,combo) cget -values] == "" } {
	    # if { [ TTKComboBoxCGet $ChangeVarInfo(AsianFont,combo) -values] == "" }
		set lst_fnts [ lsort [ GiD_Fonts get AsianFonts]]
		set ChangeVarInfo(AsianFonts) $lst_fnts
		set values ""
		foreach ifnt $lst_fnts {
		    #lappend values "[ lindex $ifnt 0]: [ lindex $ifnt 1] [ lindex $ifnt 2]"
		    lappend values "[ lindex $ifnt 0]: [ lindex $ifnt 1]"
		}
		$ChangeVarInfo(AsianFont,combo) configure -values $values
		# TTKComboBoxConfigure $ChangeVarInfo(AsianFont,combo) -values $values
	    }
	
	    set cur_fnt [ GiD_Fonts get Current]
	    set idioma [ lindex $cur_fnt 0]
	    set nombre [ lindex $cur_fnt 1]
	    # set estilo [ lindex $cur_fnt 2]
	    #set ChangeVarInfo(AsianFont,family) "${idioma}: $nombre $estilo"
	    set ChangeVarInfo(AsianFont,family) "${idioma}: $nombre"
	    #set ChangeVarInfo(AsianFont,size) [ lindex $cur_fnt 3]
	}

	GidUtils::EndWaitState .gid
	
    } elseif { $ChangeVarInfo(type) == "FORMAT" } {

	foreach fmt "PostLegendFormat PostScalarFormat PostVectorFormat PostMatrixFormat PostDefaultRealFormat" {
	    set ChangeVarInfo($fmt) [ GiD_Set $fmt]
	}

	set ChangeVarInfo(RealFormat,Default) $GidPriv(RealFormat,Default)

    } elseif { $ChangeVarInfo(type) == "GRID" } {
	set ChangeVarInfo(Grid,Show) [GiD_Set Grid(Show)]
	set ChangeVarInfo(Grid,Active) [GiD_Set Grid(Active)]
	set ChangeVarInfo(Grid,SpacingX) [GiD_Set Grid(SpacingX)]
	set ChangeVarInfo(Grid,SpacingY) [GiD_Set Grid(SpacingY)]
	set ChangeVarInfo(Grid,MajorLineEvery) [GiD_Set Grid(MajorLineEvery)]
	set ChangeVarInfo(Grid,ExtentsX) [GiD_Set Grid(ExtentsX)]
	set ChangeVarInfo(Grid,ExtentsY) [GiD_Set Grid(ExtentsY)]
	set ChangeVarInfo(Grid,DrawAxis) [GiD_Set Grid(DrawAxis)]
	set ChangeVarInfo(Grid,CenterX) [GiD_Set Grid(CenterX)]
	set ChangeVarInfo(Grid,CenterY) [GiD_Set Grid(CenterY)]
	set ChangeVarInfo(Grid,DrawLabels) [GiD_Set Grid(DrawLabels)]
	ActualizaCaja x
	ActualizaCaja y
    } elseif { $ChangeVarInfo(type) == "POSTPROCESS" } {

	#foreach fmt "PostLegendFormat PostScalarFormat PostVectorFormat PostMatrixFormat PostDefaultRealFormat" {
	#    set ChangeVarInfo($fmt) [ GiD_Set $fmt]
	#}
	#
	#set GidPriv(RealFormat,Default) $GidPriv(RealFormat,Default)

	foreach var [ list OGL_useMeshObjects OGL_useCFillTexture PostUseResultCache PostMaxMemResultCache] {
	    set ::ChangeVarInfo($var) [ GiD_Set $var]
	}
    }
}


proc Rball_FillValuesDefault { } {
    global ChangeVarInfo
    #rball options
    #Algorithm to use: 
    #0=Optimization Explicit, 1=Optimization Radius, 2=Porosity Predefined, 3=Radius Expansion, 4=Optimization Radius Fast 
    set ChangeVarInfo(Rball,Implicit) 3
    set ChangeVarInfo(Rball,MaxIter) 500
    set ChangeVarInfo(Rball,MaxIterLocal) 2
    set ChangeVarInfo(Rball,Tolerance) 1.0e-15
    set ChangeVarInfo(Rball,DeltaPosition) 0.3
    set ChangeVarInfo(Rball,DeltaRadius) 0.3 
    set ChangeVarInfo(Rball,MinRadius) 0.3
    set ChangeVarInfo(Rball,MaxRadius) 1.8 
    set ChangeVarInfo(Rball,Preprocess) 1 ;#0=None,1=Non porosity, 2=Porosity 
    set ChangeVarInfo(Rball,Porosity) 0.2
    set ChangeVarInfo(Rball,Postprocess) 0
    set ChangeVarInfo(Rball,Verbose) 0
    set ChangeVarInfo(Rball,Overlap) 0
    set ChangeVarInfo(Rball,Filter) 5 ;#Filter_Iterations
    set ChangeVarInfo(Rball,Search) 5 ;#Search_Iterations        
    set ChangeVarInfo(Rball,SearchType) 1 ;#1=SpatialSearch, 2=DelaunayRemeshing        
    set ChangeVarInfo(Rball,ContactFactor) 1.25
    set ChangeVarInfo(Rball,NeighborFactor) 1.0
    set ChangeVarInfo(Rball,FacetFactor) 1.0
    set ChangeVarInfo(Rball,OverlapFactor) 0.4
}

proc Cartesian_FillValuesDefault { } {
    global ChangeVarInfo
    set ChangeVarInfo(Rball,FromRenderMesh) 0
    set ChangeVarInfo(Rball,CondFaceElem) 0
    set ChangeVarInfo(Rball,MinSizeMainGrid) 1e-8
    set ChangeVarInfo(Rball,WeightStart) 0.0
    set ChangeVarInfo(Rball,WeightEnd) 0.0
    set ChangeVarInfo(Rball,GridUniform) 0    
}

proc FillValuesDefault {} {
    global ChangeVarInfo

    if { $ChangeVarInfo(type) == "GENERAL" } {
	set ChangeVarInfo(Dialogbrowser) 1
	set ChangeVarInfo(SplashWindow) 1
	set ChangeVarInfo(ViewMenu) 1
	set ChangeVarInfo(Createnewpoint) Ask
	set ChangeVarInfo(Displaycoordinates) 1
	set ChangeVarInfo(AutomaticRedraw) "All windows"
	set ChangeVarInfo(Usemorewindows) yes
	set ChangeVarInfo(MaxNumOfRecentProjects) 5
	set ChangeVarInfo(MaxNumOfRecentPostFiles) 8
	set ChangeVarInfo(MaxNumOfRecentViewFiles) 8
	set ChangeVarInfo(ReloadLastModel) 0
	set ChangeVarInfo(GID_OMP_NUM_THREADS) 1        
	set ChangeVarInfo(LoadMeshingPrefOfModel) 0
	set ChangeVarInfo(Sound) 0
	set ChangeVarInfo(SmallWinSelecting) 0
	set ChangeVarInfo(WriteBatchCheck) 0
	set ChangeVarInfo(WriteBatchFile) ""
	set ChangeVarInfo(BackupTime) 20
	set ChangeVarInfo(BackupCheck) 1
	set ChangeVarInfo(WriteRotationsInBatch) 0
	#is best to find the local language, not fix english
	set ChangeVarInfo(Language) "en"
    } elseif { $ChangeVarInfo(type) == "GRAPH" } {
	set ChangeVarInfo(SoftwareOpenGL) 1
	set ChangeVarInfo(OGL_emulateFrontBuffer) 0
	set ChangeVarInfo(Lightsmoothedelements) 1
	set ChangeVarInfo(AngleSmoothed) 40
	set ChangeVarInfo(DrawSurfaceMode) "Boundary lines"
	set ChangeVarInfo(DrawQuadratic) No
	set ChangeVarInfo(LabelOptions) 1
	set ChangeVarInfo(NLightingSubdiv) 8
	set ChangeVarInfo(CurvePrecision) 1.0
	set ChangeVarInfo(SurfacePrecision) 1.0
	set ChangeVarInfo(FastRotation) 0
	set ChangeVarInfo(AutomaticRotationCenter) 1
	
	set ChangeVarInfo(FRPoints) 1
	set ChangeVarInfo(FRLines) 1
	set ChangeVarInfo(FRSurfaces) 1
	set ChangeVarInfo(FRVolumes) 1
	set ChangeVarInfo(AlwaysGeom) 0
	set ChangeVarInfo(FRCurvePrecision) 0
	set ChangeVarInfo(DrawGraphicObjects) 1
	set ChangeVarInfo(BackgroundColor) [GiDColorToTkColor [GiD_Info RGBDefaultBackground]]
	set ChangeVarInfo(ColorPoints) "#000000"
	set ChangeVarInfo(ColorLines) "#0000ff"
	set ChangeVarInfo(ColorPolyLines) "#5f8f2f"
	set ChangeVarInfo(ColorSurfaces) "#ff00ff"
	set ChangeVarInfo(ColorSurfsIsoparametric) "#ffff00"
	set ChangeVarInfo(ColorVolumes) "#00ffff"
	set ChangeVarInfo(ColorNodes) "#000000"
	set ChangeVarInfo(ColorElements) "#5f8f2f"
	# 1 color algo ms apagado
	# 0 normal (color standard windows)
	set ChangeVarInfo(GUIAppearance) 1
	# 0: fondo normal y liso    1: fondo degradado entre dos colores
	set ChangeVarInfo(BackColorType) 0
	ChangeColorEnableDisableWidgets
	set ChangeVarInfo(BackColorTop) \#6699ff
	set ChangeVarInfo(BackColorBottom) \#cccc99
	ChangeColorFondoGiD BackColorTop 1
	ChangeColorFondoGiD BackColorBottom 1
	set ChangeVarInfo(BackColorSense) d
	ChangeColorSenseFondoGiD $ChangeVarInfo(BackColorSense)
    } elseif { $ChangeVarInfo(type) == "MESHING" } {
	
	set ChangeVarInfo(SizeTransitions) 0.6
	set ChangeVarInfo(BoundaryWeightedTransition) 1
	set ChangeVarInfo(QuadraticType) 0
	set ChangeVarInfo(SurfaceMesher) 0
	set ChangeVarInfo(VolumeMesher) 0
	set ChangeVarInfo(MeshUntilEnd) 1
	set ChangeVarInfo(SymmetricalStructuredTriangles) 1
	set ChangeVarInfo(SymmetricalStructuredTetrahedra) 1
	set ChangeVarInfo(AvoidElementsInBoundary) 0
	set ChangeVarInfo(AllowAutomaticStructured) 0
	set ChangeVarInfo(NoMeshFrozenLayers) 1
	set ChangeVarInfo(AutomaticCorrectSizes) 2
	set ChangeVarInfo(ForceMeshPoints) 0
	set ChangeVarInfo(ForceMeshLines) 0
	set ChangeVarInfo(ForceMeshSurfaces) 0
	set ChangeVarInfo(HighQualitySmoothing) Normal
	set ChangeVarInfo(Rjump,TakeCareCurvatures) 100.0
	set ChangeVarInfo(Rjump,TakeCareLayers) 0
	set ChangeVarInfo(Rjump,TangencyLimitAngle) 10.0
	set ChangeVarInfo(Rjump,IfRjumpFailsMeshSeparately) 1
	set ChangeVarInfo(ChordalError,ApplyChordalErrorToModel) 0
	set ChangeVarInfo(ChordalError,RelativeChordalError) 0.
	set ChangeVarInfo(ChordalError,MaxChordalError) 0.
	set ChangeVarInfo(ChordalError,MinElemSize) 0.
	set ChangeVarInfo(BoundaryLayer,GrowLaw) 0
	set ChangeVarInfo(BoundaryLayer,GrowFactor) 2.
	set ChangeVarInfo(BoundaryLayer,AllowPrismaticElements) 0
	set ChangeVarInfo(BoundaryLayer,MeshInSpecificLayer) 0
	set ChangeVarInfo(TypeOfQuadMesher) 1
	set ChangeVarInfo(AllowQuadTriMeshes) 0
	set ChangeVarInfo(GID_OMP_NUM_THREADS) 1
	set ChangeVarInfo(VMeshIso,AlphaShort) 0.
	set ChangeVarInfo(VMeshIso,AlphaLong) 0.        
	
	Rball_FillValuesDefault        
	Cartesian_FillValuesDefault
	
    } elseif { $ChangeVarInfo(type) == "EXCHANGE" } {
	
	set ChangeVarInfo(AutoCollapseAfterImport) 1
	set ChangeVarInfo(ImportTolerance) 1.0e-4
	set ChangeVarInfo(CollapseIgnoringLayers) 1
	set ChangeVarInfo(IGESIgnoreInvisible) 1
	set ChangeVarInfo(IGESSolidsManifoldBRep) 1
	set ChangeVarInfo(IGESCreateAllInLayerToUse) 0
	set ChangeVarInfo(IGESCurveOnSurfaceFrom3D) 1
	set ChangeVarInfo(AutoImportTolerance) 1

    }  elseif { $ChangeVarInfo(type) == "FONTS" } {
	font delete NormalFont FixedFont BoldFont BigFont SmallFont
	SetFontsDefault

	# the Comments font
	#set com_font [ GiD_Set CommentsFont]
	#lo anterior da el actual, no un valor por defecto!!
	set com_font {Helvetica 12}
	if { [ lsearch [ font names] CommentsFont] == -1 } {
	    font create CommentsFont
	}
	eval [ concat font conf CommentsFont [ font actual $com_font]]

	#set com_font [ GiD_Set LabelsFont] ;
	#lo anterior da el actual, no un valor por defecto!!
	set com_font {FixedSys 8}
	if { [ lsearch [ font names] LabelsFont] == -1 } {
	    font create LabelsFont
	}
	eval [ concat font conf LabelsFont [ font actual $com_font]]

	foreach i "NormalFont FixedFont BigFont CommentsFont LabelsFont" {
	    foreach item {family size weight slant} {
		set ChangeVarInfo($i,$item) [font conf $i -$item]
	    }
	}

	set cur_fnt [ GiD_Fonts get Current]
	set idioma [ lindex $cur_fnt 0]
	set nombre [ lindex $cur_fnt 1]
	#set estilo [ lindex $cur_fnt 2]
	#set ChangeVarInfo(AsianFont,family) "${idioma}: $nombre $estilo"
	set ChangeVarInfo(AsianFont,family) "${idioma}: $nombre"
	#set ChangeVarInfo(AsianFont,size) [ lindex $cur_fnt 3]

    } elseif { $ChangeVarInfo(type) == "FORMAT" } {
	set ChangeVarInfo(PostLegendFormat) "%.5g"
	set ChangeVarInfo(PostScalarFormat) "%.5g"
	set ChangeVarInfo(PostVectorFormat) "%.4g"
	set ChangeVarInfo(PostMatrixFormat) "%.4g"
	set ChangeVarInfo(PostDefaultRealFormat) "%.3g"
	set ChangeVarInfo(RealFormat,Default) "%.4g"
    } elseif { $ChangeVarInfo(type) == "GRID" } {
	set ChangeVarInfo(Grid,Show) 0
	set ChangeVarInfo(Grid,Active) 0
	set ChangeVarInfo(Grid,SpacingX) 1.0
	set ChangeVarInfo(Grid,SpacingY) 1.0
	set ChangeVarInfo(Grid,MajorLineEvery) 10
	set ChangeVarInfo(Grid,ExtentsX) 15.0
	set ChangeVarInfo(Grid,ExtentsY) 10.0
	set ChangeVarInfo(Grid,DrawAxis) 0
	set ChangeVarInfo(Grid,CenterX) 0.0
	set ChangeVarInfo(Grid,CenterY) 0.0
	set ChangeVarInfo(Grid,DrawLabels) 0
	ActualizaCaja x
	ActualizaCaja y
    } elseif { $ChangeVarInfo(type) == "POSTPROCESS" } {
	#set postprefs [GiD_Info postprocess get results_options]
	#
	## the default values:
	#set postncols [ lindex $postprefs 0]
	#set ChangeVarInfo(NumColors) [ lindex $postncols 0]
	#set ChangeVarInfo(IntvWidth) [ lindex $postprefs 1]
	#set postcontlimits [ lindex $postprefs 2]
	#set ChangeVarInfo(ContourLimMin) [ lindex $postcontlimits 0]
	#set ChangeVarInfo(ContourLimMax) [ lindex $postcontlimits 1]
	#set postvoff [ lindex $postprefs 4]
	#set ChangeVarInfo(VectorOffset) [ lindex $postvoff 0]
	#
	#set borderangles [ join [ GiD_Info postprocess get border_criteria]]
	#set ChangeVarInfo(BordeAngle) [ lindex $borderangles 0]
	#
	#set postvcols [ lindex $postprefs 6]
	#set ChangeVarInfo(VectNumColors) [ lindex $postvcols 0]

	foreach var [ list OGL_useMeshObjects OGL_useCFillTexture PostUseResultCache PostMaxMemResultCache] \
	    def [ list 0] {
	    set ::ChangeVarInfo($var) $def
	}
    }
}

proc UpdateWidgetsLanguage { } {
    global GidPriv
    .gid.phantom.display3dbutton delete 0 end ;#to re-create the 3rd button menu
    set modo [ GiD_Info Project ViewMode]
    set what PRE
    if { ( $modo eq "POSTUSE") || ( $modo eq "GRAPHUSE")} {
	set what POST
    }
    SetAllMenusVariables $what
    #CreateTopMenus

    ChangeTopMenusAndWindows
    ChangeVariables
    if { [info exists GidPriv(InitWindowList)] } {
	foreach geomwinname $GidPriv(InitWindowList) {
	    if { [lsearch {PrePostTopMenuWindow} $geomwinname] != "-1" } {
		continue
	    }
	    if { [info exists GidPriv($geomwinname)] && [winfo exists $GidPriv($geomwinname)] } {
		catch { eval $GidPriv($geomwinname,InitComm) }
	    }
	}
    }
    #lower command
    if { [info exists GidPriv(PrePostEntryWindowGeom)] } {
	BottomEntryFrame [lindex $GidPriv(PrePostEntryWindowGeom) 0]
    }
    ChangeWindowTitle
    DWInitUserDataOptions
}

proc Rball_AcceptValues { } {
    global ChangeVarInfo
    set res ""
    set RballVars { Implicit MaxIter MaxIterLocal Tolerance DeltaPosition DeltaRadius \
		        MinRadius MaxRadius Preprocess Porosity Postprocess \
		        Verbose Overlap Filter Search SearchType ContactFactor \
		        NeighborFactor FacetFactor OverlapFactor }
    foreach item $RballVars {
	if { $ChangeVarInfo(Rball,$item) != [GiD_Set Rball($item)] } {
	    lappend res Rball($item) $ChangeVarInfo(Rball,$item)
	}
    }
    if { $res != "" } {
	GiD_Process MEscape Utilities Variables {*}$res Mescape
	set label [_ "Meshing"]
	ChangeWarnLineVar [_ "Accepted new %s preferences" $label]
    }
}

proc AcceptValues { w } {
    global ChangeVarInfo GidPriv tcl_platform GIDDEFAULTTCL
    MakeNoise changesareaccepted
    set res ""

    if { ![info exists ChangeVarInfo(type)] } return

    set needsRestart 0

    if { $ChangeVarInfo(type) == "GENERAL" } {
	if { $ChangeVarInfo(WriteBatchCheck) == 1 && \
		$ChangeVarInfo(WriteBatchFile)  == "" } {
	    tk_dialogRAM $w.tempwin [_ "Error window"] \
		[_ "If batch file is on, one file name must be given"] error 0 [_ "OK"]
	    return
	}

	#activate trace function
	set ChangeVarInfo(BackupCheck) $ChangeVarInfo(BackupCheck)

	if { $ChangeVarInfo(Dialogbrowser) != \
		[GiD_Set DialogWrowser] } {
	    lappend res DialogWrowser $ChangeVarInfo(Dialogbrowser)
	}

	if { $ChangeVarInfo(SplashWindow) != \
		[GiD_Set SplashWindow] } {
	    lappend res SplashWindow $ChangeVarInfo(SplashWindow)
	}

	switch $ChangeVarInfo(Createnewpoint) {
	    Always {
		set rr 1
	    }
	    Ask {
		set rr 0
	    }
	    Never {
		set rr -2
	    }
	    No {
		set rr -1
	    }
	    default {
		# problems
		set rr -3
	    }
	}
	if { $rr != [GiD_Set CreateAlwaysNewPoint] } {
	    lappend res CreateAlwaysNewPoint $rr
	}
	if { $ChangeVarInfo(Displaycoordinates) != \
		[GiD_Set DisplayCoordinates] } {
	    lappend res DisplayCoordinates $ChangeVarInfo(Displaycoordinates)
	    UpdateCoordFrameView $ChangeVarInfo(Displaycoordinates)
	    # if { !$ChangeVarInfo(Displaycoordinates) } {
	    #         pack forget .gid.comm.fcoord
	    # } else {
	    #         pack forget .gid.comm.fle
	    #         pack forget .gid.comm.fwarn
	    #         pack .gid.comm.fcoord -fill y -side right
	    #         pack .gid.comm.fle -fill x -side bottom -expand yes -pady 2 -padx 2
	    #         pack .gid.comm.fwarn -fill both -side top -expand yes
	    # }
	}
	
	switch $ChangeVarInfo(AutomaticRedraw) {
	    "All windows" {
		set rr 1
	    }
	    "Current window" {
		set rr 2
	    }
	    "None" {
		set rr 0
	    }
	}
	if { $rr != [GiD_Set AutomaticRedraw] } {
	    lappend res AutomaticRedraw $rr
	}

	switch $ChangeVarInfo(Usemorewindows) {
	    normal {
		set rr 1
	    }
	    beginner {
		set rr 2
	    }
	    default {
		#set rr 0
		set rr 1
	    }
	}
	if { $rr != [GiD_Set UseMoreWindows] } {
	    lappend res UseMoreWindows $rr
	}
	
	if { $ChangeVarInfo(MaxNumOfRecentProjects) != \
	    [ GiD_Set MaxNumOfRecentProjects] } {
	    lappend res MaxNumOfRecentProjects $ChangeVarInfo(MaxNumOfRecentProjects)
	}
	if { $ChangeVarInfo(MaxNumOfRecentPostFiles) != \
	    [ GiD_Set MaxNumOfRecentPostFiles] } {
	    lappend res MaxNumOfRecentPostFiles $ChangeVarInfo(MaxNumOfRecentPostFiles)
	}
	if { $ChangeVarInfo(MaxNumOfRecentViewFiles) != \
	    [ GiD_Set MaxNumOfRecentViewFiles] } {
	    lappend res MaxNumOfRecentViewFiles $ChangeVarInfo(MaxNumOfRecentViewFiles)
	}
	if { $ChangeVarInfo(ReloadLastModel) != \
	    [GiD_Set ReloadLastModel] } {
	    lappend res ReloadLastModel $ChangeVarInfo(ReloadLastModel)
	}
	
	if { $ChangeVarInfo(GID_OMP_NUM_THREADS) != \
	    [GiD_Set GID_OMP_NUM_THREADS] } {
	    lappend res GID_OMP_NUM_THREADS $ChangeVarInfo(GID_OMP_NUM_THREADS)
	}
		        
	if { $ChangeVarInfo(LoadMeshingPrefOfModel) != \
	    [GiD_Set LoadMeshingPrefOfModel] } {
	    lappend res LoadMeshingPrefOfModel $ChangeVarInfo(LoadMeshingPrefOfModel)
	}
	if { $ChangeVarInfo(Sound) != \
	    [GiD_Set Sound] } {
	    lappend res Sound $ChangeVarInfo(Sound)
	}
	if { $ChangeVarInfo(SmallWinSelecting) != \
	    [GiD_Set SmallWinSelecting] } {
	    lappend res SmallWinSelecting $ChangeVarInfo(SmallWinSelecting)
	}
	
	regsub -all {\\} $ChangeVarInfo(WriteBatchFile) {/} \
		ChangeVarInfo(WriteBatchFile)

	set input [lindex [GiD_Set BatchFileToWrite] 0]
	if { $input == "UNKNOWN" && $ChangeVarInfo(WriteBatchCheck)} {
	    lappend res BatchFileToWrite $ChangeVarInfo(WriteBatchFile)
	} elseif { $input != "UNKNOWN" && !$ChangeVarInfo(WriteBatchCheck) } {
	    lappend res BatchFileToWrite UNKNOWN
	} elseif { $input != "UNKNOWN" && $ChangeVarInfo(WriteBatchCheck) && \
		$input != $ChangeVarInfo(WriteBatchFile) } {
	    lappend res BatchFileToWrite $ChangeVarInfo(WriteBatchFile)
	}

	set input [GiD_Set BackUpMinutes]
	if { $input != $ChangeVarInfo(BackupTime) } {
	    lappend res BackUpMinutes $ChangeVarInfo(BackupTime)
	}

	if { $ChangeVarInfo(WriteRotationsInBatch) != \
		[GiD_Set WriteRotationsInBatch] } {
	    lappend res WriteRotationsInBatch $ChangeVarInfo(WriteRotationsInBatch)
	}
	
	if { $ChangeVarInfo(Language) != [::GidUtils::GetCurrentLanguage] } {
	    #WarnWin [_ "Must restart %s to use the new language" $GidPriv(ProgName)]
	    ::GidUtils::SetCurrentLanguage $ChangeVarInfo(Language) 1
	    #WarnWin [_ "Some problems may appear until the program is restarted."]
	}

    }

    if { $res != "" } {
	GiD_Process MEscape Utilities Variables {*}$res Mescape
	set label [_ "General"]
	ChangeWarnLineVar [_ "Accepted new %s preferences" $label]
	set res ""
    }

    if { $ChangeVarInfo(type) == "GRAPH" } {
	if { $ChangeVarInfo(SoftwareOpenGL) != $GidPriv(SoftwareOpenGL) } {
	    if { $ChangeVarInfo(SoftwareOpenGL) == 1 } {
		# WarnWin [_ "Must restart %s to use OpenGL by software" $::GidPriv(ProgName)]
		WarnWinTk [_ "%s must be restarted in order to use OpenGL by software" $::GidPriv(ProgName)] $w
	    } else {
		# WarnWin [_ "Must restart %s to use OpenGL by hardware" $::GidPriv(ProgName)]
		WarnWinTk [_ "%s must be restarted in order to use OpenGL by hardware" $::GidPriv(ProgName)] $w
	    }
	    set GidPriv(SoftwareOpenGL) $ChangeVarInfo(SoftwareOpenGL)
	    set needsRestart 1
	}

	if { $ChangeVarInfo(OGL_emulateFrontBuffer) != [ GiD_Set OGL_emulateFrontBuffer]} {
	    lappend res OGL_emulateFrontBuffer $ChangeVarInfo(OGL_emulateFrontBuffer)
	}

	if { $ChangeVarInfo(Lightsmoothedelements) != \
		[GiD_Set LightSmoothedElems] } {
	    lappend res LightSmoothedElems $ChangeVarInfo(Lightsmoothedelements)
	}
	set CosAngleSmoothed [format %g [expr cos($ChangeVarInfo(AngleSmoothed)*\
		0.017453292519943295)]]
	set CosAngleSmoothedS [format %g [GiD_Set CosSmoothedElems]]
	if { $CosAngleSmoothed != $CosAngleSmoothedS } {
	    lappend res CosSmoothedElems $CosAngleSmoothed
	}
	
	switch $ChangeVarInfo(DrawSurfaceMode) {
	    "None" { set var 0 }
	    "Boundary lines" { set var 1 }
	    "Isoparametric lines" { set var 2 }
	    "Both modes" { set var 3 }
	}
	if { [GiD_Set DrawSurfaceMode] != $var } {
	    lappend res DrawSurfaceMode $var
	}

	if { $ChangeVarInfo(DrawQuadratic) != [GiD_Set DrawQuadraticElements]} {
	    lappend res DrawQuadraticElements $ChangeVarInfo(DrawQuadratic)
	}

	if { $ChangeVarInfo(LabelOptions) != [GiD_Set LabelOptions]} {
	    lappend res LabelOptions $ChangeVarInfo(LabelOptions)
	}

	if { $ChangeVarInfo(CurvePrecision) != \
		[GiD_Set CurvePrecision] } {
	    lappend res CurvePrecision $ChangeVarInfo(CurvePrecision)
	}

	if { $ChangeVarInfo(SurfacePrecision) != \
		[GiD_Set SurfacePrecision] } {
	    lappend res SurfacePrecision $ChangeVarInfo(SurfacePrecision)
	}

	foreach i {BackgroundColor ColorPoints ColorLines ColorPolyLines \
		ColorSurfaces ColorSurfsIsoparametric ColorVolumes \
		ColorNodes ColorElements} {
	    set bgColor [TkColorToGiDColor $ChangeVarInfo($i)]
	    if { $bgColor != [GiD_Set $i] } {
		lappend res $i $bgColor
	    }
	}

	if { $ChangeVarInfo(GUIAppearance) != $GidPriv(GUIAppearance) } {
	    if { $ChangeVarInfo(GUIAppearance) == 0 } {
		WarnWin [_ "Must restart %s to update the change\
		         of GUI appearance to standard color" $GidPriv(ProgName)]
	    } else {
		WarnWin [_ "Must restart %s to update the change\
		        of GUI appearance to attenuated color" $GidPriv(ProgName)]
	    }
	    set GidPriv(GUIAppearance) $ChangeVarInfo(GUIAppearance)
	}

	foreach bc "BackColorType BackColorTop BackColorBottom BackColorSense" {
	    if { $ChangeVarInfo($bc) != \
		     [GiD_Set $bc] } {
		lappend res $bc $ChangeVarInfo($bc)
	    }
	}
	
	if { $ChangeVarInfo(FastRotation) != \
		[GiD_Set FastRotation(Active)] } {
	    lappend res FastRotation(Active) $ChangeVarInfo(FastRotation)
	}
	if { $ChangeVarInfo(AutomaticRotationCenter) != \
		 [ GiD_Set AutomaticRotationCenter] } {
	    lappend res AutomaticRotationCenter $ChangeVarInfo(AutomaticRotationCenter)
	}
	if { $ChangeVarInfo(FRPoints) != \
		[GiD_Set FastRotation(Points)] } {
	    lappend res FastRotation(Points) $ChangeVarInfo(FRPoints)
	}
	if { $ChangeVarInfo(FRLines) != \
		[GiD_Set FastRotation(Lines)] } {
	    lappend res FastRotation(Lines) $ChangeVarInfo(FRLines)
	}
	if { $ChangeVarInfo(FRSurfaces) != \
		[GiD_Set FastRotation(Surfaces)] } {
	    lappend res FastRotation(Surfaces) $ChangeVarInfo(FRSurfaces)
	}
	if { $ChangeVarInfo(FRVolumes) != \
		[GiD_Set FastRotation(Volumes)] } {
	    lappend res FastRotation(Volumes) $ChangeVarInfo(FRVolumes)
	}
	if { [GiD_Set FastRotation(CurvePrecision)] != "" && \
		$ChangeVarInfo(FRCurvePrecision) != \
		[GiD_Set FastRotation(CurvePrecision)] } {
	    lappend res FastRotation(CurvePrecision) $ChangeVarInfo(FRCurvePrecision)
	}
	if { $ChangeVarInfo(AlwaysGeom) != \
		[GiD_Set FastRotation(UseAlwaysGeom)] } {
	    lappend res FastRotation(UseAlwaysGeom) $ChangeVarInfo(AlwaysGeom)
	}
	if { $ChangeVarInfo(DrawGraphicObjects) != \
		[GiD_Set FastRotation(DrawGraphicObjects)] } {
	    lappend res FastRotation(DrawGraphicObjects) $ChangeVarInfo(DrawGraphicObjects)
	}
    }

    if { $res != "" } {
	GiD_Process MEscape Utilities Variables {*}$res Mescape
	set label [_ "Graphical"]
	ChangeWarnLineVar [_ "Accepted new %s preferences" $label]
	set res ""
    }

    # if { ![info exists ChangeVarInfo(type)] } return

    if { $ChangeVarInfo(type) == "MESHING" } {

	if { $ChangeVarInfo(SizeTransitions) != \
		[GiD_Set SizeTransitionsFactor] } {
	    lappend res SizeTransitionsFactor $ChangeVarInfo(SizeTransitions)
	}
	
	if { $ChangeVarInfo(BoundaryWeightedTransition) != \
	    [GiD_Set BoundaryWeightedTransition] } {
	    lappend res BoundaryWeightedTransition $ChangeVarInfo(BoundaryWeightedTransition)
	}

	
	if { $ChangeVarInfo(QuadraticType) != \
	    [GiD_Set QuadraticType] } {
	    lappend res QuadraticType $ChangeVarInfo(QuadraticType)
	}
	
	if { $ChangeVarInfo(SurfaceMesher) != \
		[GiD_Set SurfaceMesher] } {
	    lappend res SurfaceMesher $ChangeVarInfo(SurfaceMesher)
	}

	if { $ChangeVarInfo(VolumeMesher) != \
		[GiD_Set VolumeMesher] } {
	    lappend res VolumeMesher $ChangeVarInfo(VolumeMesher)
	}
	
	if { $ChangeVarInfo(MeshUntilEnd) != \
		[GiD_Set MeshUntilEnd] } {
	    lappend res MeshUntilEnd $ChangeVarInfo(MeshUntilEnd)
	}
	if { $ChangeVarInfo(SymmetricalStructuredTriangles) != \
	    [GiD_Set SymmetricalStructuredTriangles] } {
	    lappend res SymmetricalStructuredTriangles $ChangeVarInfo(SymmetricalStructuredTriangles)
	}
	if { $ChangeVarInfo(AvoidElementsInBoundary) != \
	    [GiD_Set AvoidElementsInBoundary] } {
	    lappend res AvoidElementsInBoundary $ChangeVarInfo(AvoidElementsInBoundary)
	}
	if { $ChangeVarInfo(SymmetricalStructuredTetrahedra) != \
	    [GiD_Set SymmetricalStructuredTetrahedra] } {
	    lappend res SymmetricalStructuredTetrahedra $ChangeVarInfo(SymmetricalStructuredTetrahedra)
	}
	if { $ChangeVarInfo(AllowAutomaticStructured) != \
	    [GiD_Set AllowAutomaticStructured] } {
	    lappend res AllowAutomaticStructured $ChangeVarInfo(AllowAutomaticStructured)
	}

	if { $ChangeVarInfo(NoMeshFrozenLayers) != \
		[GiD_Set NoMeshFrozenLayers] } {
	    lappend res NoMeshFrozenLayers $ChangeVarInfo(NoMeshFrozenLayers)
	}
	
	if { $ChangeVarInfo(AutomaticCorrectSizes) != \
	    [GiD_Set AutomaticCorrectSizes] } {
	    lappend res AutomaticCorrectSizes $ChangeVarInfo(AutomaticCorrectSizes)
	}
	set ForceMeshEntities 0
	if { $ChangeVarInfo(ForceMeshPoints) } {
	    set ForceMeshEntities [expr $ForceMeshEntities +1]
	}
	if { $ChangeVarInfo(ForceMeshLines) } {
	    set ForceMeshEntities [expr $ForceMeshEntities +2]
	}
	if { $ChangeVarInfo(ForceMeshSurfaces) } {
	    set ForceMeshEntities [expr $ForceMeshEntities +4]
	}
	if { $ForceMeshEntities != \
	    [GiD_Set ForceMeshEntities] } {
	    lappend res ForceMeshEntities $ForceMeshEntities
	}
	
	if { $ChangeVarInfo(HighQualitySmoothing)=="Normal" } {
	    set HighQualitySmoothing 0
	} elseif { $ChangeVarInfo(HighQualitySmoothing)=="HighAngle" } {
	    set HighQualitySmoothing 1
	} else {
	    set HighQualitySmoothing 2
	}
	if { $HighQualitySmoothing != [GiD_Set HighQualitySmoothing] } {
	    lappend res HighQualitySmoothing $HighQualitySmoothing
	}

	set RjumpVars { TakeCareCurvatures TakeCareLayers TangencyLimitAngle IfRjumpFailsMeshSeparately}
	foreach item $RjumpVars {
	    if { $ChangeVarInfo(Rjump,$item) != [GiD_Set Rjump($item)] } {
		lappend res Rjump($item) $ChangeVarInfo(Rjump,$item)
	    }
	}

	set BoundaryLayerVars { GrowLaw GrowFactor AllowPrismaticElements MeshInSpecificLayer}
	foreach item $BoundaryLayerVars {
	    if { $ChangeVarInfo(BoundaryLayer,$item) != [GiD_Set BoundaryLayer($item)] } {
		lappend res BoundaryLayer($item) $ChangeVarInfo(BoundaryLayer,$item)
	    }
	}

	set ChordalErrorVars { ApplyChordalErrorToModel RelativeChordalError MaxChordalError MinElemSize }
	foreach item $ChordalErrorVars {
	    if { $ChangeVarInfo(ChordalError,$item) != [GiD_Set ChordalError($item)] } {
		lappend res ChordalError($item) $ChangeVarInfo(ChordalError,$item)
	    }
	}
		        
	set VMeshIsoVars { AlphaShort AlphaLong }
	foreach item $VMeshIsoVars {
	    if { $ChangeVarInfo(VMeshIso,$item) != [GiD_Set VMeshIso($item)] } {
		lappend res VMeshIso($item) $ChangeVarInfo(VMeshIso,$item)
	    }
	}                                
	
	if { $ChangeVarInfo(TypeOfQuadMesher) != [GiD_Set TypeOfQuadMesher] } {
	    lappend res TypeOfQuadMesher $ChangeVarInfo(TypeOfQuadMesher)
	}
	
	if { $ChangeVarInfo(GID_OMP_NUM_THREADS) != [GiD_Set GID_OMP_NUM_THREADS] } {
	    lappend res GID_OMP_NUM_THREADS $ChangeVarInfo(GID_OMP_NUM_THREADS)
	}
	

	if { $ChangeVarInfo(AllowQuadTriMeshes) != [GiD_Set AllowQuadTriMeshes] } {
	    lappend res AllowQuadTriMeshes $ChangeVarInfo(AllowQuadTriMeshes)
	}

	foreach item { Implicit MaxIter } {
	    if { $ChangeVarInfo(Rball,$item) != [GiD_Set Rball($item)] } {
		lappend res Rball($item) $ChangeVarInfo(Rball,$item)
	    }
	}

	set CartesianVars { FromRenderMesh CondFaceElem MinSizeMainGrid WeightStart WeightEnd GridUniform }
	foreach item $CartesianVars {
	    if { $ChangeVarInfo(Cartesian,$item) != [GiD_Set Cartesian($item)] } {
		lappend res Cartesian($item) $ChangeVarInfo(Cartesian,$item)
	    }
	}
	

	if { $res != "" } {
	    GiD_Process MEscape Utilities Variables {*}$res Mescape
	    set label [_ "Meshing"]
	    ChangeWarnLineVar [_ "Accepted new %s preferences" $label]
	    set res ""
	}
    }

    set res ""
    if { $ChangeVarInfo(type) == "EXCHANGE" } {

	if { $ChangeVarInfo(AutoCollapseAfterImport) != \
		[GiD_Set AutoCollapseAfterImport] } {
	    lappend res AutoCollapseAfterImport $ChangeVarInfo(AutoCollapseAfterImport)
	}

	if { $ChangeVarInfo(ImportTolerance) != \
		[GiD_Set ImportTolerance] } {
	    lappend res ImportTolerance $ChangeVarInfo(ImportTolerance)
	}

	if { $ChangeVarInfo(CollapseIgnoringLayers) != \
		[GiD_Set CollapseIgnoringLayers] } {
	    lappend res CollapseIgnoringLayers $ChangeVarInfo(CollapseIgnoringLayers)
	}

	if { $ChangeVarInfo(IGESIgnoreInvisible) != \
		[GiD_Set IGESIgnoreInvisible] } {
	    lappend res IGESIgnoreInvisible $ChangeVarInfo(IGESIgnoreInvisible)
	}

	if { $ChangeVarInfo(IGESSolidsManifoldBRep) != \
		[GiD_Set IGESSolidsManifoldBRep] } {
	    lappend res IGESSolidsManifoldBRep $ChangeVarInfo(IGESSolidsManifoldBRep)
	}

	if { $ChangeVarInfo(IGESCreateAllInLayerToUse) != \
		[GiD_Set IGESCreateAllInLayerToUse] } {
	    lappend res IGESCreateAllInLayerToUse $ChangeVarInfo(IGESCreateAllInLayerToUse)
	}

	if { $ChangeVarInfo(IGESCurveOnSurfaceFrom3D) != \
		[GiD_Set IGESCurveOnSurfaceFrom3D] } {
	    lappend res IGESCurveOnSurfaceFrom3D $ChangeVarInfo(IGESCurveOnSurfaceFrom3D)
	}

	if { $ChangeVarInfo(AutoImportTolerance) != \
		[GiD_Set AutoImportTolerance] } {
	    lappend res AutoImportTolerance $ChangeVarInfo(AutoImportTolerance)
	}

	if { $res != "" } {
	    GiD_Process MEscape Utilities Variables {*}$res Mescape
	    set label [_ "Import"]
	    ChangeWarnLineVar [_ "Accepted new %s preferences" $label]
	    set res ""
	}
    }
    if { $ChangeVarInfo(type) == "FONTS" } {
	# las fuentes de menus, ventanas, etc.
	foreach i "NormalFont FixedFont BigFont BoldFont" {
	    set fv $i
	    if { $i == "BoldFont" } {
		set fv NormalFont
		set weight bold
	    } else {
		set weight $ChangeVarInfo($fv,weight)
	    }
	    font configure $i -family $ChangeVarInfo($fv,family) \
		-size $ChangeVarInfo($fv,size) \
		-weight $weight \
		-slant $ChangeVarInfo($fv,slant)
	}
	
	# ahora las dependientes de pre y post
	set modo [ lindex [ GiD_Info Project] 4]
	set esPost 0
	if { ( $modo eq "POSTUSE") || ( $modo eq "GRAPHUSE")} {
	    set esPost 1
	}
	if { $esPost} {
	    set lst_fnts "PGF_DefaultFont PGF_AxisFont PGF_LegendFont PGF_LabelFont PGF_AsianFont PGF_TkDrawOGLFont"
	    foreach if $lst_fnts {
		AcceptPGFGiDFont $if
	    }
	    AcceptPGFModoRender
	} else {
	    foreach i "CommentsFont LabelsFont" {
		set fv $i
		if { $i == "BoldFont" } {
		    set fv NormalFont
		    set weight bold
		} else {
		    set weight $ChangeVarInfo($fv,weight)
		}
		font configure $i -family $ChangeVarInfo($fv,family) \
		    -size $ChangeVarInfo($fv,size) \
		    -weight $weight \
		    -slant $ChangeVarInfo($fv,slant)
	    }
	
	    AcceptGiDFont CommentsFont
	    AcceptGiDFont LabelsFont
	    # WarnWin "AcceptAsianFont $ChangeVarInfo(AsianFont,family)"
	    AcceptAsianFont $ChangeVarInfo(AsianFont,family)
	}
    }
    set res ""
    if { $ChangeVarInfo(type) == "FORMAT" } {
	set formatchanges 0
	foreach fmt "PostLegendFormat PostScalarFormat PostVectorFormat PostMatrixFormat PostDefaultRealFormat" {
	    if { $ChangeVarInfo($fmt) != [GiD_Set $fmt]} {
		GiD_Process escape escape escape escape utilities variables $fmt \
		        $ChangeVarInfo($fmt) escape escape escape escape
		set formatchanges 1
	    }
	}

	if { $GidPriv(RealFormat,Default) != $ChangeVarInfo(RealFormat,Default) } {
	    set GidPriv(RealFormat,Default) $ChangeVarInfo(RealFormat,Default)
	    set formatchanges 1
	}

	if { $formatchanges } {
	    set label [_ "Format"]
	    ChangeWarnLineVar [_ "Accepted new %s preferences" $label]
	}

    }
    set res ""
    if { $ChangeVarInfo(type) == "GRID" } {
	if { $ChangeVarInfo(Grid,Show) != [GiD_Set Grid(Show)] } {
	    lappend res Grid(Show) $ChangeVarInfo(Grid,Show)
	    set ::GidPriv(Grid,Show) $ChangeVarInfo(Grid,Show) ;#to update button and redraw
	}
	if { $ChangeVarInfo(Grid,Active) != [GiD_Set Grid(Active)] } {
	    lappend res Grid(Active) $ChangeVarInfo(Grid,Active)
	    set ::GidPriv(Grid,Active) $ChangeVarInfo(Grid,Active) ;#to update button
	}
	if { $ChangeVarInfo(Grid,SpacingX) != [GiD_Set Grid(SpacingX)] } {
	    lappend res Grid(SpacingX) $ChangeVarInfo(Grid,SpacingX)
	}
	if { $ChangeVarInfo(Grid,SpacingY) != [GiD_Set Grid(SpacingY)] } {
	    lappend res Grid(SpacingY) $ChangeVarInfo(Grid,SpacingY)
	}
	if { $ChangeVarInfo(Grid,MajorLineEvery) != [GiD_Set Grid(MajorLineEvery)] } {
	    lappend res Grid(MajorLineEvery) $ChangeVarInfo(Grid,MajorLineEvery)
	}
	if { $ChangeVarInfo(Grid,ExtentsX) != [GiD_Set Grid(ExtentsX)] } {
	    lappend res Grid(ExtentsX) $ChangeVarInfo(Grid,ExtentsX)
	}
	if { $ChangeVarInfo(Grid,ExtentsY) != [GiD_Set Grid(ExtentsY)] } {
	    lappend res Grid(ExtentsY) $ChangeVarInfo(Grid,ExtentsY)
	}
	if { $ChangeVarInfo(Grid,DrawAxis) != [GiD_Set Grid(DrawAxis)] } {
	    lappend res Grid(DrawAxis) $ChangeVarInfo(Grid,DrawAxis)
	}
	if { $ChangeVarInfo(Grid,CenterX) != [GiD_Set Grid(CenterX)] } {
	    lappend res Grid(CenterX) $ChangeVarInfo(Grid,CenterX)
	}
	if { $ChangeVarInfo(Grid,CenterY) != [GiD_Set Grid(CenterY)] } {
	    lappend res Grid(CenterY) $ChangeVarInfo(Grid,CenterY)
	}
	if { $ChangeVarInfo(Grid,DrawLabels) != [GiD_Set Grid(DrawLabels)] } {
	    lappend res Grid(DrawLabels) $ChangeVarInfo(Grid,DrawLabels)
	}
	if { $res != "" } {
	    GiD_Process MEscape Utilities Variables {*}$res Mescape
	    set label [_ "Grid"]
	    ChangeWarnLineVar [_ "Accepted new %s preferences" $label]
	    set res ""
	}
    }

    if { $ChangeVarInfo(type) == "POSTPROCESS" } {
	set res ""
	foreach var [ list OGL_useMeshObjects OGL_useCFillTexture PostUseResultCache PostMaxMemResultCache] {
	    if { $::ChangeVarInfo($var) != [ GiD_Set $var]} {
		lappend res $var $::ChangeVarInfo($var)
	    }
	}
	
	if { $res != "" } {
	    GiD_Process MEscape Utilities Variables {*}$res Mescape
	    set label [_ "Postprocess"]
	    ChangeWarnLineVar [_ "Accepted new %s preferences" $label]
	    set res ""
	}
    }

    if { $needsRestart} {
	RestartGiD
    }
}


proc DynamicPrecision { val } {
    GiD_Set CurvePrecision $val
    GiD_Redraw
}

proc ChangeToPrefType { w name } {
    global ChangeVarInfo GidPriv
    $w conf -cursor watch
    update
    switch $name {
	General {
	    set ChangeVarInfo(type) GENERAL
	}
	Graphical {
	    set ChangeVarInfo(type) GRAPH
	}
	Meshing {
	    set ChangeVarInfo(type) MESHING
	}
	Exchange {
	    set ChangeVarInfo(type) EXCHANGE
	}
	Fonts {
	    set ChangeVarInfo(type) FONTS
	}
	Format {
	    set ChangeVarInfo(type) FORMAT
	}
	Grid {
	    set ChangeVarInfo(type) GRID
	}
	Postprocess {
	    set ChangeVarInfo(type) POSTPROCESS

	    #get_all_post_limits
	}
    }

    FillPrefValuesFromGid
    $w conf -cursor ""
    set GidPriv(CurrentPreference) $name
}

proc get_all_post_limits { } {
    global ChangeVarInfo

    set postprefs [GiD_Info postprocess get results_options]

    set postncols [ lindex $postprefs 0]
    set ChangeVarInfo(MinColors) [ lindex $postncols 1]
#    set ChangeVarInfo(MaxColors) [ lindex $postncols 2]

    set postconttype [ lindex $postprefs 3]
    set ChangeVarInfo(ContourAllTypes) "[ lindex $postconttype 1] [ lindex $postconttype 2]"

    set postvoff [ lindex $postprefs 4]
    set ChangeVarInfo(MinVectorOffset) [ lindex $postvoff 1]
    set ChangeVarInfo(MaxVectorOffset) [ lindex $postvoff 2]

    set postvcol [ lindex $postprefs 5]
    set ChangeVarInfo(VectorAllColors) "[ lindex $postvcol 1] [ lindex $postvcol 2]"

    set base $ChangeVarInfo(BasePostWin)
    set existPost [GiD_Info Project ExistPost]

    set borderangles [ join [ GiD_Info postprocess get border_criteria]]
    set ChangeVarInfo(BordeAngleMin) [ lindex $borderangles 1]
    set ChangeVarInfo(BordeAngleMax) [ lindex $borderangles 2]

    set postvcols [ lindex $postprefs 6]
    set ChangeVarInfo(VectMinColors) [ lindex $postvcols 1]
#    set ChangeVarInfo(VectMaxColors) [ lindex $postvcols 2]

}

#
# ChangeVariables
#



source $GIDDEFAULTTCL/pref_general.ui.tcl
source $GIDDEFAULTTCL/pref_graphical.ui.tcl
source $GIDDEFAULTTCL/pref_format.ui.tcl
source $GIDDEFAULTTCL/pref_meshing.ui.tcl
source $GIDDEFAULTTCL/pref_import.ui.tcl
source $GIDDEFAULTTCL/pref_fonts_ui.tcl
source $GIDDEFAULTTCL/pref_grid.ui.tcl
source $GIDDEFAULTTCL/pref_postprocess.ui.tcl

#curpref can be: General Graphical Meshing Exchange Fonts  Format Grid or Postprocess
proc ChangeVariables { { w .gid.wvar } {curpref ""} } {
    global GidPriv

    set existPost [GiD_Info Project ExistPost]

    InitWindow $w [_ "Preferences"] ChangeVariablesWindowGeom ChangeVariables
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
	
    wm withdraw $w

    if { $curpref == "" } {
	if { [info exists GidPriv(CurrentPreference)] } {
	    set curpref $GidPriv(CurrentPreference)
	} else {
	    set curpref General
	}
    }

    frame $w.buts -bg [CCColorActivo [$w cget -background]]
    ttk::button $w.buts.ok -text [_ "Accept"] -command "AcceptValues $w" -underline 0
    ttk::button $w.buts.rs -text [_ "Reset"] -command FillValuesDefault -underline 0
    ttk::button $w.buts.cn -text [_ "Close"] -command "destroy $w" -underline 0

    #SetWidgetsWidthFromText $w.buts.ok $w.buts.rs $w.buts.cn

    #pack $w.buts.ok $w.buts.rs $w.buts.cn -side left -padx 5 -ipadx 5 -pady 5
    grid $w.buts.ok -row 0 -column 0 -padx 5 -pady 5
    grid $w.buts.rs -row 0 -column 1 -padx 5 -pady 5
    grid $w.buts.cn -row 0 -column 2 -padx 5 -pady 5

    GidHelp $w.buts.ok [_ "With this button, changed values of the active tab are\
    accepted. If this button is not pressed, values changed by the user are not\
    computed."]
    GidHelp $w.buts.rs [_ "This button sets all the options of the active tab to its\
	    default values."]
    GidHelp $w.buts.cn [_ "This button closes the window without accepting any changed\
	    value. Press 'Accept' not to loose the changed options."]

    focus $w.buts.ok

    frame $w.f
    grid $w.buts -row 1 -column 0 -sticky nsew
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }
    grid $w.f -row 0 -column 0 -sticky nsew
    grid rowconfigure $w.f 0 -weight 1
    grid columnconfigure $w.f 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1

    set commands "pref_general_ui pref_graphical_ui pref_meshing_ui \
		pref_exchange_ui pref_fonts_ui pref_format_ui pref_grid_ui pref_postprocess_ui"
    set names { "General" "Graphical" "Meshing" "Exchange" "Fonts" "Format" "Grid" "Postprocess"}
    set labels [list [_ "General"]  [_ "Graphical"]  [_ "Meshing"]  \
		        [_ "Exchange"]  [_ "Fonts"] [_ "Format"] [_ "Grid"] [_ "Postprocess"]]
    # set commands "pref_general_ui pref_graphical_ui pref_meshing_ui \
    #             pref_exchange_ui pref_fonts_ui pref_format_ui pref_grid_ui"
    # set names { "General" "Graphical" "Meshing" "Exchange" "Fonts" "Format" "Grid"}
    # set labels [list [_ "General"]  [_ "Graphical"]  [_ "Meshing"]  \
    #                     [_ "Exchange"]  [_ "Fonts"] [_ "Format"] [_ "Grid"]]

    set def_back [$w cget -background]
    NoteBook $w.f.nb -internalborderwidth 0 -activebackground [CCColorActivo $def_back]
    foreach command $commands name $names label $labels {
	$command [$w.f.nb insert end $name -text $label]
    }
    pack $w.f.nb -fill both -expand yes

    $w.f.nb bindtabs <1> [list ChangeToPrefType $w]

    update idletasks

    ChangeToPrefType $w $curpref
    $w.f.nb raise $curpref

    wm deicon $w

    bind $w <Escape> "event generate $w.buts.cn <<Invoke>>"
    bind $w <Alt-a>  "event generate $w.buts.ok <<Invoke>>"
    bind $w <Alt-r>  "event generate $w.buts.rs <<Invoke>>"
    bind $w <Alt-c>  "event generate $w.buts.cn <<Invoke>> ; break"

    bind $w <Destroy> [list +DestroyChangeVar %W $w] ;# + to add to previous script
}

proc DestroyChangeVar { W w } {
    if { $W != $w } return
    global ChangeVarInfo
    array unset ChangeVarInfo ;#destroy also traces
}
