# dialog-ram.tcl --
#
# This file defines the procedure tk_dialog, which creates a dialog
# box containing an image, a message, and one or more buttons.
#
# SCCS: @(#) dialog.tcl 1.25 96/04/10 15:43:33
#
# Copyright (c) 1992-1993 The Regents of the University of California.
# Copyright (c) 1994-1996 Sun Microsystems, Inc.
#
# See the file "license.terms" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#


proc ReplaceImageName { image } {
    if { $image == "gidquestionhead" || $image == "questhead" || $image == "question"  } {
	set image question-32.png
    } elseif { $image == "info" } {
	set image info-32.png        
    } elseif { $image == "error" } {
	set image error-32.png
    } elseif { $image == "warning" } {
	set image warning-32.png
    }
    return $image
}

#
# tk_dialog:
#
# This procedure displays a dialog box, waits for a button in the dialog
# to be invoked, then returns the index of the selected button.
#
# Arguments:
# w -                Window to use for dialog top-level.
# title -        Title to display in dialog's decorative frame.
# text -        Message to display in dialog.
# image -       Image to display in dialog (empty string means none).
# default -        Index of button that is to display the default ring
#                (-1 means none). (-2 means 0 and button 2 returns -2)
# args -        One or more strings to display in buttons across the
#                bottom of the dialog box.

proc tk_dialogRAM {w title text image default args} {
    tk_dialogRAM2 $w $title $text "" $image $default {*}$args
}

#tk_dialogRAMFull prototype only for back compatibility, bitmap is now an ignored dummy field
#was invoked for example by nastran 3.2.4 or tdyn5.4
proc tk_dialogRAMFull {w title text textsmall dummy image default args} {
    tk_dialogRAM2 $w $title $text $textsmall $image $default {*}$args
}

proc tk_dialogRAM2 {w title text textsmall image default args} {
    global tkPriv tcl_platform GidPriv
    
    set image [ReplaceImageName $image]
    
    if { [winfo exists .gid.central.s] && ( [.gid.central.s disable windows] || [GiD_Set usemorewindows] == 0 ) } {
	return
    }
    if { $w == "" } {
	if { [winfo exists .gid] } {
	    set w .gid.__dialogram
	} else {
	    set w .__dialogram
	}
    }
    
    # 1. Create the top-level window and divide it into top
    # and bottom parts.
   
    while { [winfo exists $w] } { append w a }
    
    toplevel $w -class OverlayWindow
    if { $::tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
	wm attributes $w -toolwindow 1
    }

    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW [list event generate $w <Escape>]

    if { $default == -2 } {
	set default 0
	bind $w <2> "set tkPriv(button) -2"
	bind $w <Control-x><Control-l> "set tkPriv(button) -2"
    }

    if { [llength $args] > 6 } {
	frame $w.bot3 -relief raised -bd 1
	pack $w.bot3 -side bottom -fill both
    }
    if { [llength $args] > 3 } {
	frame $w.bot2 -relief raised -bd 1
	pack $w.bot2 -side bottom -fill both
    }
    #frame $w.bot -relief raised -bd 1
    frame $w.bot -bg [CCColorActivo [$w cget -background]]
    pack $w.bot -side bottom -fill both

    if { $textsmall != "" } {
	ttk::label $w.ts -text $textsmall -font SmallFont
	pack $w.ts -side bottom -anchor w
    }

    #frame $w.top -relief raised -bd 0
    ttk::frame $w.top

    pack $w.top -side top -fill both -expand 1

    # 2. Fill the top part with image and message (use the option
    # database for -wraplength so that it can be overridden by
    # the caller).

#    option add *Dialog.msg.wrapLength 3i widgetDefault
    ttk::label $w.msg -justify left -text $text -wraplength 75m -font BigFont

    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if {$image != ""} {
	ttk::label $w.bitmap -image [GetImage $image]
	pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
    }    

    # 3. Create a row of buttons at the bottom of the dialog.

    set buttonslist ""
    set UnderLineList ""
    set i 0
    foreach but $args {
	regsub -all {([a-z0-9])([A-Z])} $but {\1 \2} but
    }
    foreach but $args {
	regsub -all {([a-z0-9])([A-Z])} $but {\1 \2} but
	ttk::button $w.button$i -text $but -command [list set tkPriv(button) $i]
	lappend buttonslist $w.button$i
	if { $i < 3 } {
	    pack $w.button$i -in $w.bot -side left -expand 1 -padx 3m -pady 2m
	} elseif { $i < 6 }  {
	    pack $w.button$i -in $w.bot2 -side left -expand 1 -padx 3m -pady 2m
	} else {
	    pack $w.button$i -in $w.bot3 -side left -expand 1 -padx 3m -pady 2m
	}

	bind $w.button$i <Return> "
	    update idletasks
	    after 100
	    set tkPriv(button) $i
	"
	set lsize [string length $but]
	for {set j 0} {$j < $lsize} {incr j} {
	    set ichar [string tolower [string index $but $j]]
	    if { [lsearch -exact $UnderLineList $ichar] == -1 } {
		lappend UnderLineList $ichar
		$w.button$i conf -underline $j
	      # this catch is to deal with extrange language, like korean
	      catch {
		bind $w <KeyPress-[string toupper $ichar]> "
		   update idletasks
		   after 100
		   set tkPriv(button) $i
		"
		bind $w <KeyPress-[string tolower $ichar]> "
		   update idletasks
		   after 100
		   set tkPriv(button) $i
		"
	      }

		break
	    }
	}

	incr i
    }

    if { [llength $buttonslist] > 0 } {
      #set escape binding to right button
      bind $w <Escape> "
      update idletasks
      after 100
      set tkPriv(button) [expr [llength $buttonslist]-1]
      "

      #set buttons size acordind to current text
      #SetWidgetsWidthFromText {*}$buttonslist
    }

    # 4. Create a binding for <Return> on the dialog if there is a
    # default button.

    # 5. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

    if { [winfo parent $w] == ".gid" } {
	bind $w <Destroy> [list +DestroyWinInDialogs $w GeomDialog]
    }
    if { [info exists GidPriv(GeomDialog)] && [winfo parent $w] == ".gid" } {
	WmGidGeom $w $GidPriv(GeomDialog)
    } else {
	wm withdraw $w
	update idletasks
	
	# extrange errors with updates
	if { ![winfo exists $w] } { return 0}
	if { $w != ".gid.__dialogram" } {
	    set pare [winfo toplevel [winfo parent $w]]
	    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
	    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
	} else {
	    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
		    - [winfo vrootx [winfo parent $w]]]
	    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
		    - [winfo vrooty [winfo parent $w]]]
	}
	if { $x < 0 } { set x 0 }
	if { $y < 0 } { set y 0 }
	WmGidGeom $w +$x+$y
	update
	wm deiconify $w
    }

    # 6. Set a grab and claim the focus too.

    set oldFocus [focus]
    if { $::tcl_platform(os) != "Darwin"} {
	tkwait visibility $w
    } else {
	update
    }
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    if {$default >= 0} {
	focus $w.button$default
	after 100 catch [list "focus -force $w.button$default"]

    } else {
	focus $w
    }

    # 7. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.

    tkwait variable tkPriv(button)

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }
    return $tkPriv(button)
}

proc tk_dialogRAMHelp {w title text image HelpCommand default args} {
    global tkPriv tcl_platform GidPriv
	
    set image [ReplaceImageName $image]
    
    if { $w == "" } {
	if { [winfo exists .gid] } {
	    set w .gid.__dialogram
	} else { set w .__dialogram }
    }

    # 1. Create the top-level window and divide it into top
    # and bottom parts.

    while { [winfo exists $w] } { append w a }

    catch {destroy $w}
    toplevel $w -class OverlayWindow
    if { $::tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
	wm attributes $w -toolwindow 1
    }

    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW [list event generate $w <Escape>]

    if { $default == -2 } {
	set default 0
	bind $w <2> "set tkPriv(button) -2"
    }

    if { [llength $args] > 6 } {
	frame $w.bot3 -relief raised -bd 1
	pack $w.bot3 -side bottom -fill both
    }
    if { [llength $args] > 3 } {
	frame $w.bot2 -relief raised -bd 1
	pack $w.bot2 -side bottom -fill both
    }
    frame $w.bot -relief raised -bd 1 -bg [CCColorActivo [$w cget -background]]
    pack $w.bot -side bottom -fill both
    frame $w.top -relief raised -bd 1
    pack $w.top -side top -fill both -expand 1

    # 2. Fill the top part with image and message (use the option
    # database for -wraplength so that it can be overridden by
    # the caller).

#    option add *Dialog.msg.wrapLength 3i widgetDefault
    ttk::label $w.msg -justify left -text $text -wraplength 75m -font BigFont

    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if { $image != ""} {
	ttk::button $w.bitmap -image [GetImage $image] -command $HelpCommand        
	pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m -ipadx 2m -ipady 2m
    }

    # 3. Create a row of buttons at the bottom of the dialog.

    set buttonslist ""
    set UnderLineList ""
    set i 0
    foreach but $args {
	regsub -all {([a-z0-9])([A-Z])} $but {\1 \2} but
    }
    foreach but $args {
	regsub -all {([a-z0-9])([A-Z])} $but {\1 \2} but
	ttk::button $w.button$i -text $but -command [list set tkPriv(button) $i]
	lappend buttonslist $w.button$i
	if { $i < 3 } {
	    pack $w.button$i -in $w.bot -side left -expand 1 -padx 3m -pady 2m
	} elseif { $i < 6 }  {
	    pack $w.button$i -in $w.bot2 -side left -expand 1 -padx 3m -pady 2m
	} else {
	    pack $w.button$i -in $w.bot3 -side left -expand 1 -padx 3m -pady 2m
	}
	bind $w.button$i <Return> "
	    update idletasks
	    after 100
	    set tkPriv(button) $i
	"
	if { [regexp -nocase cancel $but] || [regexp -nocase escape $but] || [regexp -nocase close $but] } "
	    bind $w <Escape> \"
	    update idletasks
	    after 100
	    set tkPriv(button) $i
	\""
	set lsize [string length $but]
	for {set j 0} {$j < $lsize} {incr j} {
	    set ichar [string tolower [string index $but $j]]
	    if { [lsearch -exact $UnderLineList $ichar] == -1 } {
		lappend UnderLineList $ichar
		$w.button$i conf -underline $j
	      # this catch is to deal with extrange language, like korean
	      catch {
		bind $w <KeyPress-[string toupper $ichar]> "
		   update idletasks
		   after 100
		   set tkPriv(button) $i
		"
		bind $w <KeyPress-[string tolower $ichar]> "
		   update idletasks
		   after 100
		   set tkPriv(button) $i
		"
	      }

		break
	    }
	}

	incr i
    }

    if { [llength $buttonslist] > 0 } {
      #set escape binding to right button
      bind $w <Escape> "
      update idletasks
      after 100
      set tkPriv(button) [expr [llength $buttonslist]-1]
      "

      #set buttons size acordind to current text
      SetWidgetsWidthFromText {*}$buttonslist
    }

    if { [winfo exists $w.bot3] } {
	set packin $w.bot3
    } elseif { [winfo exists $w.bot2] } {
	set packin $w.bot2
    } else { set packin $w.bot }
    ttk::button $w.help -text [_ "Help"] -command $HelpCommand
    pack $w.help -in $packin -side left -expand 1 \
		    -padx 3m -pady 2m


    # 4. Create a binding for <Return> on the dialog if there is a
    # default button.

    # 5. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

    if { [winfo parent $w] == ".gid" } {
	bind $w <Destroy> [list +DestroyWinInDialogs $w GeomDialog]
    }
    if { [info exists GidPriv(GeomDialog)] && [winfo parent $w] == ".gid" } {
	WmGidGeom $w $GidPriv(GeomDialog)
    } else {
	wm withdraw $w
	update idletasks
	
	# extrange errors with updates
	if { ![winfo exists $w] } { return 0}
	
	if { $w != ".gid.__dialogram" } {
	    set pare [winfo toplevel [winfo parent $w]]
	    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
	    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
	} else {
	    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
		    - [winfo vrootx [winfo parent $w]]]
	    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
		    - [winfo vrooty [winfo parent $w]]]
	}
	if { $x < 0 } { set x 0 }
	if { $y < 0 } { set y 0 }
	WmGidGeom $w +$x+$y
	update
	wm deiconify $w
    }

    # 6. Set a grab and claim the focus too.

    set oldFocus [focus]
    if { $::tcl_platform(os) != "Darwin"} {
	tkwait visibility $w
    }
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    if {$default >= 0} {
	focus $w.button$default
	after 100 catch [list "focus -force $w.button$default"]

    } else {
	focus $w
    }

    # 7. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.

    tkwait variable tkPriv(button)

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }

    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    grab -global $oldGrab
	} else {
	    grab $oldGrab
	}
    }
    return $tkPriv(button)
}



proc DestroyWinInDialogs { w geomname } {
    global GidPriv

    bind $w <Destroy> ""
    regexp {[-+]+[0-9]+[-+]+[0-9]+} [wm geom $w] GidPriv($geomname)
#     set GidPriv($geomname) [wm geom $w]
}


#
# This procedure displays a dialog box, with an entry, waits for a button in
# the dialog to be invoked, then returns the entry.
# If cancel is pressed, it returns: --CANCEL--
#
# Arguments:
# w -                Window to use for dialog top-level.
# title -        Title to display in dialog's decorative frame.
# text -        Message to display in dialog.
# image -       image to display in dialog (empty string means none).
# type -        Can be:
#                        word:    word without spaces
#                        any:     anything
#                        real:    real
#                        real+:   positive real
#                        int:     int
#                        int+:    positive int
#                        text:    text with spaces and '\n'
#                        textro:  text with spaces and '\n' read-only
# default -    default value for entry
# args -        One or more strings to display in buttons across the
#                bottom of the dialog box.

proc tk_dialogEntryRAM {w title text image type default {btnstxt ""} } {
    global tkPriv GidPriv tcl_platform
    
    set image [ReplaceImageName $image]

    if { $btnstxt == "" } {
	set args [list [_ "Ok"] [_ "Cancel"]]
    } else {
	set args $btnstxt
    }

    # 1. Create the top-level window and divide it into top, middle
    # and bottom parts.

    while { [winfo exists $w] } { append w a }
    catch {destroy $w}
    toplevel $w -class OverlayWindow
    if { $::tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
	wm attributes $w -toolwindow 1
    }

    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW [list event generate $w <Escape>]

    frame $w.bot -relief raised -bd 1 -bg [CCColorActivo [$w cget -background]]
    pack $w.bot -side bottom -fill both
    frame $w.top -relief raised -bd 1

    if { $type != "text" && $type != "textro"} {
	pack $w.top -side top -fill both -expand 1
    } else {
	pack $w.top -side top -fill both
    }

    frame $w.middle -relief raised -bd 1
    if { $type != "text" && $type != "textro"} {
	ttk::entry $w.middle.e -xscroll "$w.middle.xscroll set" -textvariable ::GidPriv(entry)
	if { $type == "password" } { $w.middle.e conf -show * }

	$w.middle.e delete 0 end
	if { $default != "" } {
	    #kike,correccin si $default == "", no hara caso a la tecla retroceso
	      $w.middle.e insert end $default
	     $w.middle.e selection range 0 end
	}

	ttk::scrollbar $w.middle.xscroll -orient horizontal -command [list $w.middle.e xview]
#     pack $w.middle.xscroll -side bottom -fill x
#     pack $w.middle.e -side top

	grid $w.middle.e -sticky ew
	grid $w.middle.xscroll -sticky ew

	pack $w.middle

    } else {
	set GidPriv(entry) ""

	text $w.middle.t -relief sunken -width 40 -height 8 \
		-yscroll "$w.middle.yscroll set" -bd 1
	$w.middle.t delete 0.0 end
	$w.middle.t insert end $default
	#$w.middle.t tag add selection 0.0 end
	if { $type == "textro"} {
	    $w.middle.t configure -state disabled
	    bind $w.middle.t <1> "focus $w.middle.t"
	}

	ttk::scrollbar $w.middle.yscroll -orient vertical -command [list $w.middle.t yview]
	
	# grid $w.middle.t $w.middle.yscroll -sticky ewns -ipady 3
	#
	# grid rowconfigure $w.middle 0 -weight 1
	# grid rowconfigure $w.middle 1 -weight 1
	# grid columnconfigure $w.middle 0 -weight 1
	# grid columnconfigure $w.middle 1 -weight 0
	pack $w.middle.t -side left -expand 1 -fill both
	pack $w.middle.yscroll -expand 1 -fill y

	pack $w.middle -expand 1 -fill both
    }

    # 2. Fill the top part with image and message (use the option
    # database for -wraplength so that it can be overridden by
    # the caller).

#    option add *Dialog.msg.wrapLength 3i widgetDefault
    ttk::label $w.msg -justify left -text $text -wraplength 75m -font BigFont

    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if {$image != ""} {
	ttk::label $w.bitmap -image [GetImage $image]
	pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
    }

    if { $type != "text" && $type != "textro"} {
	bind $w <Return> "
	  update idletasks
	  after 100
	  set tkPriv(button) 0
	"
    }

    # 3. Create a row of buttons at the bottom of the dialog.

    set buttonslist ""
    set UnderLineList ""
    set i 0
    set default 0
    foreach but $args {
	regsub -all {([a-z0-9])([A-Z])} $but {\1 \2} but
    }
    foreach but $args {
	ttk::button $w.button$i -text $but -command [list set tkPriv(button) $i]
	lappend buttonslist $w.button$i
	pack $w.button$i -in $w.bot -side left -expand 1 -padx 3m -pady 2m

	bind $w.button$i <Return> "
	update idletasks
	after 100
	set tkPriv(button) $i
	break
	"
	incr i
    }

    if { [llength $buttonslist] > 0 } {
	#set escape binding to right button
	bind $w <Escape> "
	update idletasks
	after 100
	set tkPriv(button) [expr [llength $buttonslist]-1]
	"

	#set buttons size acordind to current text
	SetWidgetsWidthFromText {*}$buttonslist
    }


    # 5. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

    if { [winfo parent $w] == ".gid" } {
	bind $w <Destroy> [list +DestroyWinInDialogs $w GeomDialog]
    }

    if { [info exists GidPriv(GeomDialog)] && [winfo parent $w] == ".gid"} {
	WmGidGeom  $w $GidPriv(GeomDialog)
    } else {
	wm withdraw $w
	update idletasks
	
	
	# extrange errors with updates
	if { ![winfo exists $w] } { return 0}
	
	set pare [winfo toplevel [winfo parent $w]]
	set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
	set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
	if { $x < 0 } { set x 0 }
	if { $y < 0 } { set y 0 }
	WmGidGeom $w +$x+$y
	update
	wm deiconify $w
    }

    # 6. Set a grab and claim the focus too.

    set oldFocus [focus]
    if { $::tcl_platform(os) != "Darwin"} {
	tkwait visibility $w
    }
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w

    if { $type != "text" && $type != "textro" } {
	focus $w.middle.e
	$w.middle.e selection range 0 end
	after 100 catch [list "focus -force $w.middle.e"]
    } else {
	focus $w.button0
	after 100 catch [list "focus -force $w.button0"]
    }


#     elseif { $type == "textro" } {
#         focus $w.button0
#         after 100 catch [list "focus -force $w.button0"]
#     } else {
#         focus $w.middle.t
#         after 100 catch [list "focus -force $w.middle.t"]
#     }

    # 7. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.

    while 1 {
	tkwait variable tkPriv(button)
	if { $tkPriv(button) == 1} { break }
	if { $type != "text" && $type != "textro" } {
	    set result [$w.middle.e get]
	} else {
	    set result [ $w.middle.t get 0.0 end]
	}
	switch $type {
	    word {
		set result [string trim $result]

		# if { ![regexp {^[  ]*[^ \t]+[  ]*$} $result] } {}
		
		if { [string first { } $result] != -1 } {
		    set errorRes [_ "A single word must be entered"]
		}
	    }
	    real {
		if { ![regexp \
		        {^[  ]*([+-]?[0-9]*\.?[0-9]*([eE][+-]?[0-9]+)?)[  ]*$} \
		        $result] } {
		    set errorRes [_ "One number must be entered"]
		}
	    }
	    real+ {
		if { ![regexp \
		        {^[  ]*([+]?[0-9]*\.?[0-9]*([eE][+-]?[0-9]+)?)[  ]*$} \
		        $result] } {
		    set errorRes [_ "One positive number must be entered"]
		}
	    }
	    int {
		if { ![regexp \
		        {^[  ]*[+-]?[0-9]+[  ]*$} \
		        $result] } {
		    set errorRes [_ "One integer number must be entered"]
		}
	    }
	    int+ {
		if { ![regexp \
		        {^[  ]*[+]?[0-9]*[1-9]+[0-9]*[  ]*$} \
		        $result] } {
		    set errorRes [_ "One positive integer number must be entered"]
		}
	    }
	    text {
		set len [ string length $result]
		# para quitar el \n que anyade el get de antes al final.
		if { $len >= 2} {
		    set result [ string range $result 0 [ expr $len - 2]]
		} else {
		    #kike continue
		set result "--NULL--"
		}
	    if { ( $result == "\"\"") } {
		set result "--NULL--"
	    }

#                 if { ( $result == "\"\"") || ( $result == "\'\'") } {
#                     WarnWin "res = /$result/"
#                     regsub -all \" $result {\"} textres
#                     regsub -all \' $textres {\'} result
#                     WarnWin "res = /$result/"
#                 }
		#regsub -all \n $result {\n} textres
		#regsub -all \t $textres {\t} result
		set GidPriv(entry) $result
	    }
	}
	if { [info exists errorRes] } {
	    tk_dialogRAM $w.tempwin {Error window} $errorRes error 0 [_ "Ok"]
	    unset errorRes
	} else { break }
    }

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }
    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    grab -global $oldGrab
	} else {
	    grab $oldGrab
	}
    }
    if { $tkPriv(button) == 1} { return --CANCEL-- }
    return $GidPriv(entry)
}


#
# This procedure displays a dialog box, with a combobox, waits for a button in
# the dialog to be invoked, then returns the entry.
# If cancel is pressed, it returns: --CANCEL--
#
# Arguments:
# w -                Window to use for dialog top-level.
# title -        Title to display in dialog's decorative frame.
# text -        Message to display in dialog.
# image -       Image to display in dialog (empty string means none).
# AcceptNew     Can be: TCL_BOOLEAN or 2 (then, word must begin with a letter)
#                       3 (then, one positive number must be entered)
#                       4 any word (including spaces)
#
# args some words to add to the combo
#

proc tk_dialogComboRAM {w title text image AcceptNew args} {
    global tkPriv GidPriv tcl_platform

    set image [ReplaceImageName $image]
    
    # 1. Create the top-level window and divide it into top, middle
    # and bottom parts.

    while { [winfo exists $w] } { append w a }
    catch {destroy $w}
    toplevel $w -class OverlayWindow
    if { $::tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
	wm attributes $w -toolwindow 1
    }

    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW [list event generate $w <Escape>]

    frame $w.bot -relief raised -bd 0 -bg [CCColorActivo [$w cget -background]]
    pack $w.bot -side bottom -fill both
    # ramsan 6/02
    #frame $w.top -relief raised -bd 1
    frame $w.top -relief raised -bd 0
    pack $w.top -side top -fill both -expand 1

    frame $w.middle -relief raised -bd 0

    set GidPriv(entry) ""
    set iseditable $AcceptNew
    if { $iseditable > 1 } { set  iseditable 1 }

    ComboBox $w.middle.e -editable $iseditable \
	-textvariable GidPriv(entry)

    set last ""
    set values ""
    foreach i [lindex $args 0] {
	if { $i == $last } { continue }
	# ramsan 6/02
	lappend values $i
	#$w.middle.e add $i
	set last $i
    }
    $w.middle.e configure -values $values

    if { $AcceptNew == 0 || $AcceptNew == 3 } {
	set GidPriv(entry) [lindex [lindex $args 0] 0]
	# ramsan 6/02
#         $w.middle.e selection range 0 end
    }

    pack $w.middle.e -side top -padx 30 -fill x -expand 1 -pady 5
    pack $w.middle -fill x -expand 1 -pady 5


    # 2. Fill the top part with image and message (use the option
    # database for -wraplength so that it can be overridden by
    # the caller).

    option add *Dialog.msg.wrapLength 3i widgetDefault
    ttk::label $w.msg -justify left -text $text -wraplength 75m -font BigFont

    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if {$image != ""} {
	ttk::label $w.bitmap -image [GetImage $image]        
	pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
    }

    bind $w <Return> "
       update idletasks
       after 100
       set tkPriv(button) 0
    "

    # 3. Create a row of buttons at the bottom of the dialog.

    set buttonslist ""
    set UnderLineList ""
    set i 0

    set default 0
    set args [list [_ "Ok"] [_ "Cancel"]]
    foreach but $args {
	regsub -all {([a-z0-9])([A-Z])} $but {\1 \2} but
    }
    foreach but $args {
	ttk::button $w.button$i -text $but -command [list set tkPriv(button) $i]
	lappend buttonslist $w.button$i
	pack $w.button$i -in $w.bot -side left -expand 1 -padx 3m -pady 2m
	bind $w.button$i <Return> "
	    update idletasks
	    after 100
	    set tkPriv(button) $i
	    break
	"
	incr i
    }

    if { [llength $buttonslist] > 0 } {
	#set escape binding to right button
	bind $w <Escape> "
	update idletasks
	after 100
	set tkPriv(button) [expr [llength $buttonslist]-1]
	"

	#set buttons size acordind to current text
	SetWidgetsWidthFromText {*}$buttonslist
    }


    # 5. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

    if { [winfo parent $w] == ".gid" } {
	bind $w <Destroy> [list +DestroyWinInDialogs $w GeomDialog]
    }
    if { [info exists GidPriv(GeomDialog)] && [winfo parent $w] == ".gid" } {
	WmGidGeom $w $GidPriv(GeomDialog)
    } else {
	wm withdraw $w
	update idletasks
	# extrange errors with updates
	if { ![winfo exists $w] } { return 0}
	
	set pare [winfo toplevel [winfo parent $w]]
	set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
	set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
	if { $x < 0 } { set x 0 }
	if { $y < 0 } { set y 0 }
	WmGidGeom $w +$x+$y
	update
	wm deiconify $w
    }

    # 6. Set a grab and claim the focus too.

    set oldFocus [focus]
    if { $::tcl_platform(os) != "Darwin"} {
	tkwait visibility $w
    } else {
	update
    }
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w

    # ramsan 6/02
    focus $w.middle.e
    update idletasks
    event generate $w.middle.e <<TraverseIn>>
    #focus [$w.middle.e cget entry]
    #     after 100 catch [list "focus -force $w.middle.e"]


    # 7. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.

    while 1 {
	tkwait variable tkPriv(button)
	if { $tkPriv(button) == 1} { break }
	# ramsan 6/02
	#set result [$w.middle.e get]
	set result $GidPriv(entry)
	if { $AcceptNew != 0 && $AcceptNew != 4 } {
	    #if not editable not check is word is single
	    if { ![regexp {^[          ]*[^ ]+[  ]*$} $result] } {
		set errorRes [_ "A single word must be entered"]
	    }
	}
	if { $AcceptNew == 2 && ![regexp {^[a-zA-Z]} $result] } {
	    set errorRes [_ "Word must begin with a letter"]
	} elseif { $AcceptNew == 3 && ![regexp \
		{^[  ]*([+]?[0-9]*\.?[0-9]*([eE][+-]?[0-9]+)?)[  ]*$} \
		$result] } {
	    set errorRes [_ "One positive number must be entered"]
	}
	if { [info exists errorRes] } {
	    tk_dialogRAM $w.tempwin [_ "Error window"] $errorRes error 0 [_ "Ok"]
	    unset errorRes
	} else { break }
    }

    catch {focus $oldFocus}
    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    grab -global $oldGrab
	} else {
	    grab $oldGrab
	}
    }
    if { $tkPriv(button) == 1} { return --CANCEL-- }
    return $GidPriv(entry)
}


proc enter_link {w} {
    $w tag configure link -underline on -foreground #336699
    $w configure -cursor hand2
}


proc leave_link {w} {
    $w tag configure link -underline off -foreground #336699
    $w configure -cursor arrow
}

proc CallChooseProblemtype { button } {
  global GidPriv

  set retval [ChooseProblemtype $button]
  if { $retval != "--CANCEL--" } {
    set GidPriv(RegisterModule) $retval
    set GidPriv(RegisterModuletext) ""
    foreach i $GidPriv(RegisterModule) {
      lappend GidPriv(RegisterModuletext) [file tail [file root $i]]
    }
    if { $GidPriv(RegisterModuletext) == "" } {
      set GidPriv(RegisterModuletext) ...
    }
    GidHelp $GidPriv(RegisterModulewin) [join $GidPriv(RegisterModuletext) \n]
  }
}


proc SearchOtherPasswords { w lst_modules text entry } {

    regexp {Name:\s+(\S+)} $text {} computername

    if { !$lst_modules } {
	set title [_ "Select %s installation directory where the password is installed" $::GidPriv(ProgName)]
    } else {
	set title [_ "Select %s problemtype.gid directory where the password is installed" $::GidPriv(ProgName)]
    }
    set dir [tk_chooseDirectory -initialdir $::GIDDEFAULT -parent $w -title $title \
		 -mustexist 1]
    if { $dir == "" } { return }

    if { !$lst_modules } {
	if { ![file exists [file join $dir scripts TemporalVariables]] } {
	    WarnWin [_ "Error: You must select the directory containing one installed %s" $::GidPriv(ProgName)] $w
	    return
	}
	set fin [open [file join $dir scripts TemporalVariables] r]
	set txt [read $fin]
	close $fin
	
	set rex "(?n)^$computername\\s+(\\S+)\[ \\t\].*(?:Expiration date|Fecha caducidad|Expiration data): (.*)\\."

	set olderdate [clock seconds]
	set olderpass ""
	foreach "- pass date" [regexp -inline -all $rex $txt] {
	    if { $pass eq "IDIDIDIDIDIDIDID" } { continue }
	    foreach "m d y" $date break
	    set date [clock scan "$d $m $y"]
	    if { $date > $olderdate } {
		set olderdate $date
		set olderpass $pass
	    }
	}
	set ipserver ""
	if { $olderpass == "" } {
	    set rex {([0-9]+)[.]([0-9]+)[.]([0-9]+)[.]([0-9]+)(:[0-9]+)?}
	    if { [regexp $rex $txt dummy n1 n2 n3 n4 port] } {
		set ipserver $n1.$n2.$n3.$n4
		if { $port != "" } {
		    append ipserver :$port
		}
	    }
	}
	if { $olderpass == "" && $ipserver == ""} {
	    WarnWin [_ "Could not find a valid password in this %s" $::GidPriv(ProgName)]
	    return
	}
	
	if { $ipserver != ""} {
	    set newpass $ipserver
	} else {
	    set newpass ""
	    set i 0
	    while 1 {
		append newpass [string range $olderpass $i [expr {$i+3}]]-
		incr i 4
		if { $i >=[string length $olderpass]  } { break }
	    }
	    set newpass [string trimright $newpass -]
	}

	$entry delete 0 end
	$entry insert end $newpass
    } else {
	if { ![string equal -nocase [file extension $dir] ".gid"] } {
	    WarnWin [_ "Error: You must select the directory containing the problemtype.gid"] $w
	    return
	}
	if { ![file exists [file join $dir password.txt]] } {
	    WarnWin [_ "Error: This problemtype does not contain passwords"] $w
	    return
	}
	set fin [open [file join $dir password.txt] r]
	set txt [read $fin]
	close $fin
	
	set rex "(?n)^$computername\\s+(\\S+)"
	set olderpass ""
	foreach "- pass" [regexp -inline -all $rex $txt] {
	    set olderpass $pass
	}
	set ipserver ""
	if { $olderpass == "" } {
	    set rex {([0-9]+)[.]([0-9]+)[.]([0-9]+)[.]([0-9]+)(:[0-9]+)?}
	    if { [regexp $rex $txt dummy n1 n2 n3 n4 port] } {
		set olderpass $n1.$n2.$n3.$n4
		if { $port != "" } {
		    append olderpass :$port
		}
	    }
	}
	if { $olderpass == "" } {
	    WarnWin [_ "Could not find a valid password in this problemtype"]
	    return
	}

	$entry delete 0 end
	$entry insert end $olderpass
    }
}

proc tk_dialogEntryPASS {w title text image type default { lst_modules 0}} {
    global tkPriv GidPriv tcl_platform
    
    set image [ReplaceImageName $image]

    # 1. Create the top-level window and divide it into top, middle
    # and bottom parts.

    while { [winfo exists $w] } { append w a }
    catch {destroy $w}
    toplevel $w -class OverlayWindow
    if { $::tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
	wm attributes $w -toolwindow 1
    }

    set GidPriv(PassW) $w

    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW [list event generate $w <Escape>]

    frame $w.bot -relief flat -bd 1 -bg [CCColorActivo [$w cget -background]]
    pack $w.bot -side bottom -fill x
    frame $w.top -relief flat -bd 1
    pack $w.top -side top -fill both -expand 1

    frame $w.middle -relief raised -bd 1

    ttk::entry $w.middle.e -textvariable ::GidPriv(entry) -width 30

    ttk::button $w.middle.search -image [GetImage find_folder.gif] \
	-command [list SearchOtherPasswords $w $lst_modules $text $w.middle.e]
    GidHelp $w.middle.search [_ "Use this button to copy the password from previously installed programs"]
    
    $w.middle.e delete 0 end
    $w.middle.e insert end $default
    $w.middle.e selection range 0 end
    
    bind $w.middle.e <3> [list ContextualEntryPASS $w.middle.e %x %y %X %Y]

    if { $lst_modules} {
      set def_back [ $w cget -background]
      set realzado [ CCColorActivo $def_back]
      ttk::label $w.middle.l1 -text [_ "Choose Problem type"]:
      ttk::label $w.middle.l2 -text [_ "Enter the password"]:

      ttk::button $w.middle.om -textvariable GidPriv(RegisterModuletext)
      set GidPriv(RegisterModulewin) $w.middle.om
      if {![info exists GidPriv(RegisterModule)] \
	      || ![llength $GidPriv(RegisterModule)]} {
	if {[catch "GiD_Info problemtypepath" GidPriv(RegisterModule)]} {
	  set GidPriv(RegisterModuletext) "..."
	  set GidPriv(RegisterModule) ""
	  GidHelp $w.middle.om ...
	} else {
	  set GidPriv(RegisterModule) [list $GidPriv(RegisterModule)]
	  set GidPriv(RegisterModuletext) [GiD_Info Project ProblemType]
	  GidHelp $w.middle.om $GidPriv(RegisterModuletext)
	}
      }

      $w.middle.om configure -command "CallChooseProblemtype $w.middle.om"

      text $w.middle.msg -bd 4 -relief flat -width 20 -height 3 -wrap word -background $realzado
      $w.middle.msg insert end [_ "This password is used to register the selected Problem types.\nTo register %s, please select 'Register' in the 'Help' menu." $::GidPriv(ProgName)]
      $w.middle.msg configure -state disabled

      grid $w.middle.l1 -sticky e
      grid $w.middle.om -sticky ew -column 1 -row 0
      grid $w.middle.l2 -sticky e
      grid $w.middle.e -sticky ew -column 1 -row 1
      grid $w.middle.msg -sticky news -columnspan 3 -pady 2 -padx 2
      grid rowconfigure $w.middle 0 -weight 1
      grid rowconfigure $w.middle 1 -weight 1
      grid rowconfigure $w.middle 2 -weight 1
      grid columnconfigure $w.middle 1 -weight 1
      pack $w.middle -fill x -padx 5 -pady 5 -ipady 1 -expand 1 -anchor s

    } else {
      #pack $w.middle.e -side top
      #pack $w.middle
      #ttk::label $w.middle.l1 -text [_ "Get it from"]:
      #ttk::button $w.middle.b1 -textvariable GidPriv(PassWeb) -command [list VisitWeb $GidPriv(PassWeb)]
      #grid $w.middle.l1 $w.middle.b1 -sticky ew

      ttk::label $w.middle.l2 -text [_ "Enter the password"]:
      grid $w.middle.l2 $w.middle.e -sticky ew
      grid columnconfigure $w.middle 1 -weight 1
      pack $w.middle -fill x -padx 5 -ipady 5 -expand 1  -anchor s
    }

  grid  $w.middle.search -column 2 -row 0

  # 2. Fill the top part with image and message (use the option
  # database for -wraplength so that it can be overridden by
  # the caller).

  #    option add *Dialog.msg.wrapLength 3i widgetDefault
  #    ttk::label $w.msg -justify left -text $text -wraplength 75m


  text $w.msg -width 30 -height 10 -wrap word -font NormalFont -bd 1

  $w.msg tag bind link <1> {VisitWeb $GidPriv(PassWeb)}
  $w.msg tag configure link -underline off -foreground #336699
  $w.msg tag bind link <Any-Enter> "enter_link $w.msg"

  $w.msg tag bind link <Any-Leave> "leave_link $w.msg"

  $w.msg ins 0.0 $text
  $w.msg insert end "\n"
  $w.msg insert end [_ "or get it from"]:\n
  if { $lst_modules} {
    $w.msg insert end "\t"
    $w.msg insert end $GidPriv(ProgWeb) link
  } else {

    $w.msg insert end "\t"
    $w.msg insert end $GidPriv(PassWeb) link
  }

  $w.msg configure -state disabled
  bind $w.msg <1> "focus $w.msg"
  bind $w.msg <3> [list ContextualTextPASS $w.msg %x %y %X %Y]

  pack $w.msg -in $w.top -side top -expand 1 -fill both -padx 1m -pady 1m
  if {$image != ""} {
     ttk::label $w.bitmap -image [GetImage $image]
     #pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
  }

  bind $w <Return> "
       update idletasks
       after 100
       set tkPriv(button) 0
    "

  # 3. Create a row of buttons at the bottom of the dialog.

  set buttonslist ""
  set UnderLineList ""
  set i 0

  set default 0
  set args [list [_ "Ok"] [_ "Evaluation"] [_ "Cancel"]]

  foreach but $args {
      regsub -all {([a-z0-9])([A-Z])} $but {\1 \2} but
  }
  foreach but $args {
    ttk::button $w.button$i -text $but -command [list set tkPriv(button) $i]
    lappend buttonslist $w.button$i
    pack $w.button$i -in $w.bot -side left -expand 1 \
	-padx 3m -pady 2m -fill x
    bind $w.button$i <Return> "
	    update idletasks
	    after 100
	    set tkPriv(button) $i
	    break
	"
    incr i
  }


  if { [llength $buttonslist] > 0 } {
      #set escape binding to right button
      bind $w <Escape> "
      update idletasks
      after 100
      set tkPriv(button) [expr [llength $buttonslist]-1]
      "

      #set buttons size acordind to current text
      SetWidgetsWidthFromText {*}$buttonslist
  }

  # 5. Withdraw the window, then update all the geometry information
  # so we know how big it wants to be, then center the window in the
  # display and de-iconify it.

  wm withdraw $w
  update idletasks


  # extrange errors with updates
  if { ![winfo exists $w] } { return 0}

  set pare [winfo toplevel [winfo parent $w]]
  set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
  set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]/2]
  if { $x < 0 } { set x 0 }
  if { $y < 0 } { set y 0 }
  WmGidGeom $w +$x+$y
  update
  wm deiconify $w

  # 6. Set a grab and claim the focus too.

  set oldFocus [focus]
  set oldGrab [grab current $w]
  if {$oldGrab != ""} {
    set grabStatus [grab status $oldGrab]
  }
  grab $w

  focus $w.middle.e
  after 100 catch [list "focus -force $w.middle.e"]

  # 7. Wait for the user to respond, then restore the focus and
  # return the index of the selected button.  Restore the focus
  # before deleting the window, since otherwise the window manager
  # may take the focus away so we can't redirect it.  Finally,
  # restore any grab that was in effect.

    while 1 {
	tkwait variable tkPriv(button)
	if { $tkPriv(button) == 1} { break }
	set result [$w.middle.e get]
	switch $type {
	    word {
		if { ![regexp {^[  ]*[^ \t]+[  ]*$} $result] } {
		    set errorRes [_ "A single word must be entered"]
		}
	    }
	    real {
		if { ![regexp \
		        {^[  ]*([+-]?[0-9]*\.?[0-9]*([eE][+-]?[0-9]+)?)[  ]*$} \
		        $result] } {
		    set errorRes [_ "One number must be entered"]
		}
	    }
	    real+ {
		if { ![regexp \
		        {^[  ]*([+]?[0-9]*\.?[0-9]*([eE][+-]?[0-9]+)?)[  ]*$} \
		        $result] } {
		    set errorRes [_ "One positive number must be entered"]
		}
	    }
	    int {
		if { ![regexp \
		        {^[  ]*[+-]?[0-9]+[  ]*$} \
		        $result] } {
		    set errorRes [_ "One integer number must be entered"]
		}
	    }
	    int+ {
		if { ![regexp \
		        {^[  ]*[+]?[0-9]*[1-9]+[0-9]*[  ]*$} \
		        $result] } {
		    set errorRes [_ "One positive integer number must be entered"]
		}
	    }
	}
	if { [info exists errorRes] } {
	    tk_dialogRAM $w.tempwin {Error window} $errorRes error 0 [_ "Ok"]
	    unset errorRes
	} else {
	  break
	}
    }

    catch {focus $oldFocus}
    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    grab -global $oldGrab
	} else {
	    grab $oldGrab
	}
    }

    if { $lst_modules} {
	if { $tkPriv(button) == 2 } {
	    return --CANCEL--
	} elseif { ( $tkPriv(button) == 1) && ( $GidPriv(RegisterModule) != "") } {
	  return [list --ACADEMIC-- $GidPriv(RegisterModule)]
	} elseif { ( $tkPriv(button) == 0) } {
	  set result [list $GidPriv(entry)]
	  if {$GidPriv(RegisterModule) ne ""} {
	    lappend result $GidPriv(RegisterModule)
	  }
	  return $result
	}
    } else {
	if { $tkPriv(button) == 0} {
	    regsub -all -- {-} $GidPriv(entry) {} retval
	    regsub -all -- {\n} $retval {} retval2
	    regsub -all -- { } $retval2 {} retval
	    return $retval
	} elseif { $tkPriv(button) == 1} {
	    return --ACADEMIC--
	} elseif { $tkPriv(button) == 2} {
	    return --CANCEL--
	}
    }
}

proc GetSelection { w } {
    if {
	![catch {selection get -displayof $w -selection CLIPBOARD} txt]
    } {
	return $txt
    }
    return ""
}

proc ClipboardCopy { w } {
    if {[string match $w [selection own -displayof $w]]} {
	clipboard clear -displayof $w
	catch {
	    set txt [selection get -displayof $w]
	    clipboard append -displayof $w $txt
	}
    }
}
proc ClipboardPaste { w } {
    if {![catch {GetSelection $w} txt]} {
	$w insert insert $txt
    }
}

proc ContextualEntryPASS { lb x y X Y} {
    set menu $lb.menu
    destroy $menu
    menu $menu -tearoff 0

    $menu add command -label [_ "Copy"] -command [list ClipboardCopy $lb]
    $menu add command -label [_ "Paste"] -command [list ClipboardPaste $lb]

    tk_popup $menu $X $Y
}

proc ContextualTextPASS { lb x y X Y} {
    set menu $lb.menu
    destroy $menu
    menu $menu -tearoff 0

    #$menu add command -label [_ "Copy"] -command [list ClipboardCopy $lb]
    $menu add command -label [_ "Copy"] -command [CopyTextSelectionOrAll $lb]

    tk_popup $menu $X $Y
}

#if selection is void copy all text contents
proc CopyTextSelectionOrAll { t } {
    set txt ""
    if {[string match $t [selection own -displayof $t]]} {
	catch {
	    set txt [selection get -displayof $t]
	}
    }
    if { $txt == "" } {
	set txt [$t get 1.0 end-1c]
    }
    if {  $txt != "" } {
	clipboard clear -displayof $t
	clipboard append -displayof $t $txt
    }
}

proc WarnWinText { text { title "Warning"} { traducir 0}} {

    # Traducir el valor por defecto
    if { ( $title == "Warning") && $traducir } {
	set title [_ "Warning"]
    }

    if { [winfo exists .gid] } {
	set w .gid.warning
    } else {
	set w .warning
    }
    if { ! [winfo exists $w] } {
	if { 0 && [info procs InitWindow] != "" } {
            #not using InitWindow because disappear when changing from pre to post
	    InitWindow $w $title WarnWinTextWindowGeom
	    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
	} else {
	    toplevel $w
	    wm transient $w [winfo toplevel [winfo parent $w]]
	    if { $::tcl_platform(platform) == "windows" } {
		wm attributes $w -toolwindow 1
	    }
	    wm title $w $title
	}
	#text with scrolls only if required
	frame $w.fr
	ttk::scrollbar $w.fr.scrolly -command "$w.fr.t yview" -orient vertical
	ttk::scrollbar $w.fr.scrollx -command "$w.fr.t xview" -orient horizontal
	text $w.fr.t -font SmallFont -yscrollcommand "$w.fr.scrolly set" \
		-xscrollcommand "$w.fr.scrollx set" -wrap none
	
	grid $w.fr.t -row 1 -column 1 -sticky nsew
	grid $w.fr.scrolly -row 1 -column 2 -sticky ns
	grid $w.fr.scrollx -row 2 -column 1 -sticky ew
	grid rowconfigure $w.fr 1 -weight 1
	grid columnconfigure $w.fr 1 -weight 1
	grid $w.fr -sticky nsew -padx 5 -pady 5
	
	grid remove $w.fr.scrolly
	grid remove $w.fr.scrollx
	bind $w.fr.t <Configure> "ConfigureTextScroll $w.fr.t $w.fr.scrollx $w.fr.scrolly"

	#text must be a text or cavas widget and sx,sy scrollbar widgets
	proc ConfigureTextScroll { text sx sy } {
	    foreach i "x y" {
		if { ![info exists s${i}] || ![winfo exists [set s${i}]] } { continue }
		foreach "${i}1 ${i}2" [$text ${i}view] break
		
		if { [set ${i}1] == 0 && [set ${i}2] == 1 } {
		    after idle grid remove [set s${i}]
		} else {
		    after idle grid [set s${i}]
		}
	    }
	}
	
	#lower buttons
	set def_back [$w cget -background]
	frame $w.frmButtons -bg [CCColorActivo $def_back]
	set close_txt "Close"
	if { $traducir} {
	    set close_txt [_ "Close"]
	}
	ttk::button $w.frmButtons.btnclose -text $close_txt \
		-command "destroy $w" \
		-underline 0
	SetWidgetsWidthFromText $w.frmButtons.btnclose

	grid $w.frmButtons -sticky ews -columnspan 7
	if { $::tcl_version >= 8.5 } { grid anchor $w.frmButtons center }
	grid $w.frmButtons.btnclose -padx 5 -pady 6
	grid columnconfigure $w 0 -weight 1
	grid rowconfigure $w 0 -weight 1

	focus $w.frmButtons.btnclose
	bind $w <Alt-c> [list $w.frmButtons.btnclose invoke]
	bind $w <Escape> [list $w.frmButtons.btnclose invoke]
    }

    $w.fr.t insert end "$text\n"
    $w.fr.t see end
    ConfigureTextScroll $w.fr.t $w.fr.scrollx $w.fr.scrolly
    update
}

proc CloseWarnWinText { } {
    set w .gid.warning
    if { [winfo exists $w] } {
	destroy $w
    } else {
	set w .warning
	if { [winfo exists $w] } {
	    destroy $w
	}
    }
}

#to check the current global variable traces, interesting to detect unremoved or repeated traces
proc TestTraces { } {
    set ignorevars {EntryVar EntryVarHilit ActionsMenuVar}
    set ignorearrays {GidPriv(Grid,Show) GidPriv(Coordinates) GidPriv(Grid,Active)}
    set tracedvars {}
    set badtracedvars {}
    foreach var [info globals] {
	global $var
	set items [array names $var]
	if { $items != "" } {
	    foreach item $items {
		set traceinfo [trace info variable [set var]($item)]
		if { $traceinfo != "" } {
		    if { [lsearch $ignorearrays [set var]($item)] == -1 } {
		        lappend tracedvars [set var]($item)
		    }
		    if { [llength [lsort -unique $traceinfo] ] != [llength $traceinfo] } {
		        lappend badtracedvars [set var]($item)
		    }
		}
	    }
	} else {
	    set traceinfo [trace info variable $var]
	    if { $traceinfo != "" } {
		if { [lsearch $ignorevars $var] == -1 } {
		    lappend tracedvars $var
		}
		if { [llength [lsort -unique $traceinfo] ] != [llength $traceinfo] } {
		    lappend badtracedvars $var
		}
	    }
	}
    }
    WarnWinText "vars with trace: $tracedvars"
    if { $badtracedvars != "" } {
	WarnWinText "repeating script: $badtracedvars"
    }
}

# To Debug GiD tcl whitout a window.
proc DebugInit { { filename ""}} {
    global GidPriv
    if { $filename != ""} {
	set ::GidPriv(SetDebugFileName) $filename
    } else {
	if { ![ info exists ::GidPriv(SetDebugFileName)] } {
	    if {$tcl_platform(platform) == "unix"} {
		set tmpdir /tmp
	    } else {
		set tmpdir "$env(tmp)"
	    }
	    set ::GidPriv(SetDebugFileName) [file join $tmpdir DebugGiDTcl.log]
	}

	set filename $::GidPriv(SetDebugFileName)
    }

    #open and close to try to write and clear it
    if [catch {open $filename w} f] {
	puts stderr "Cannot open $filename: $f"
	set GidPriv(DebugFilename) ""
    } else {
	set GidPriv(DebugFilename) $filename
	close $f
	file delete $filename
    }
    set ::GidPriv(DebugInitialized) 1
    set GidPriv(DebugInitTime) [ clock seconds]
    set GidPriv(DebugLastTime) $GidPriv(DebugInitTime)
    set tstr [ clock format $GidPriv(DebugInitTime) -format "%H:%M:%S %d.%m.%Y"]
    DebugText "$tstr ----- DebugInit $filename -----"
}

# DebugPrint texto
#    imprime texto en el archivo del DebugInit

proc DebugPrint { text } {
    global GidPriv
    if { $GidPriv(Configuration) != "debug" } {
	return
    }
    if { !$::GidPriv(DebugInitialized)} {
	DebugInit
    }
    if { ![info exists GidPriv(DebugFilename)] || $GidPriv(DebugFilename) == "" } {
	return
    }
    set f [open $GidPriv(DebugFilename) a]
    puts $f $text
    close $f
}

# DebugText -time -abs -rel -level texto
#    imprime texto en el archivo del DebugInit
#           -time -> imprime el tiempo desde el DebugInit
#           -time -abs -> imprime el tiempo y fecha absolutos & desde el DebugInit
#           -time -rel -> imprime el tiempo desde el ultimo -time -rel & desde el DebugInit
#           -level -> identa el mensaje segun el nivel de pila: num_espacios = 2 * [ info level]

proc DebugText { args} {
    global GidPriv

    if { $GidPriv(Configuration) != "debug" } {
	return
    }
    if { !$::GidPriv(DebugInitialized)} {
	DebugInit
    }
    if { ![info exists GidPriv(DebugFilename)] || $GidPriv(DebugFilename) == "" } {
	return
    }
    set txt ""
    set inf_time 0
    set inf_abs_time 0
    set inf_rel_time 0
    set inf_level 0
    foreach a $args {
	if { [string index $a 0] == "-"} {
	    set a [ string range $a 1 end]
	    if { ![ string compare -nocase $a "time"]} {
		set inf_time 1
	    } elseif { ![ string compare -nocase -length 3 $a "abs"]} {
		set inf_abs_time 1
	    } elseif { ![ string compare -nocase -length 3 $a "rel"]} {
		set inf_rel_time 1
	    } elseif { ![ string compare -nocase $a "level"]} {
		set inf_level 1
	    }
	} else {
	    if { "$txt" != ""} {
		append txt " $a"
	    } else {
		set txt $a
	    }
	}
    }

    set time_str ""
    if { $inf_time} {
	set tt [ clock seconds]

	# el -3600 es para que al haecr el format con 0 segundos quede 00:00:00
	set dd [ expr $tt - $GidPriv(DebugInitTime) -3600]
	set time_str [ clock format $dd -format "%H:%M:%S"]

	if { $inf_abs_time} {
	    set tstr [ clock format $tt -format "%H:%M:%S %d.%m.%Y"]
	    append time_str " $tstr"
	}

	if { $inf_rel_time} {
	    set dd [ expr $tt - $GidPriv(DebugLastTime) -3600]
	    set dstr [ clock format $dd -format "%H:%M:%S"]
	    append time_str " $dstr"
	    set GidPriv(DebugLastTime) $tt
	}
	append time_str " "
    }

    set level_str ""
    if { $inf_level} {
	set num_espacios [ expr 2 * [ info level]]
	set identacion [ string repeat " " $num_espacios]
	set level_str $identacion
    }

    DebugPrint "$time_str$level_str$txt"
}

#proc DebugTextLevel { text} {
#    set num_espacios [ expr 2 * [ info level]]
#    set identacion [ string repeat " " $num_espacios]
#    DebugText "$identacion$text"
#}
#
#proc DebugAbsTextTime { text } {
#    global GidPriv
#    set tt [ clock seconds]
#    set tstr [ clock format $tt -format "%H:%M:%S %d.%m.%Y"]
#    DebugText "$tstr   $text"
#}
#
#proc DebugTextTime { text } {
#    global GidPriv
#    set tt [ clock seconds]
#    set tstr [ clock format $tt -format "%H:%M:%S %d.%m.%Y"]
#    # el -3600 es para que al haecr el format con 0 segundos quede 00:00:00
#    set dd [ expr $tt - $GidPriv(DebugInitTime) -3600]
#    set dstr [ clock format $dd -format "%H:%M:%S"]
#    #DebugText "$tstr $dstr   $text"
#    DebugText "$dstr   $text"
#}
#
#proc DebugTextTimeLevel { text} {
#    global GidPriv
#    set tt [ clock seconds]
#    set tstr [ clock format $tt -format "%H:%M:%S %d.%m.%Y"]
#    # el -3600 es para que al haecr el format con 0 segundos quede 00:00:00
#    set dd [ expr $tt - $GidPriv(DebugInitTime) - 3600]
#    set dstr [ clock format $dd -format "%H:%M:%S"]
#    set num_espacios [ expr 2 * [ info level]]
#    set identacion [ string repeat " " $num_espacios]
#    #DebugText "$tstr $dstr $identacion$text"
#    DebugText "$dstr $identacion$text"
#}

namespace eval TipWin {
    variable _mostrar
    variable _wid
    variable _id        1
    variable _imagen
}

proc TipWin::Close { title_org} {
    variable _wid

    regsub -all { } $title_org {_} pp
    regsub -all {\.} $pp {_} title

    if { ![ info exists _wid($title)] || ![ winfo exists $_wid($title)]} {
	return
    }

    destroy $_wid($title)

}

proc TipWin::CambiaMostrar { title_org w} {
    variable _mostrar

    regsub -all { } $title_org {_} pp
    regsub -all {\.} $pp {_} title

    if { $_mostrar($title)} {
	set _mostrar($title) 0
    } else {
	set _mostrar($title) 1
    }
    $w toggle
}

proc TipWin::CreateWindow { par show_cb title_org args} {
    variable _mostrar
    variable _wid
    variable _id

    regsub -all { } $title_org {_} pp
    regsub -all {\.} $pp {_} title

    if { ![ info exists _mostrar($title)] } {
	set _mostrar($title) 1
    }
    if { !$_mostrar($title)} {
	return
    } else {
	if { [ info exists _wid($title)] && [ winfo exists $_wid($title)]} {
	    raise $_wid($title)
	    return
	}
    }

    if { "$par" == "."} {
	     set w .__Tip_Win$_id
    } else {
	set w $par.__Tip_Win$_id
    }
    incr _id

    set _wid($title) $w

    toplevel $w
    if { $::tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
	wm attributes $w -toolwindow 1
    }

    wm title $w $title_org

    set def_back [ $w cget -background]
    frame $w.f -bd 0

    set max_w 0
    set txt_w 0
    set txt_h 1
    set txt [ join $args]
    set n_txt [ string length $txt]
    for { set i 0} { $i < $n_txt} { incr i} {
	     incr txt_w
	     if { "[ string index $txt $i]" == "\n"} {
		 incr txt_h
		 if { $txt_w > $max_w} { set max_w $txt_w}
		 set txt_w 0
	     }
    }
    if { $txt_w > $max_w} { set max_w $txt_w}

    text $w.f.t -bd 4 -relief flat -wrap word -background [ CCColorActivo $def_back] \
	    -width $max_w -height $txt_h
    $w.f.t insert end $txt
    $w.f.t configure -state disabled
    grid $w.f.t -sticky nwes -padx 4 -pady 8

    if { $show_cb} {
	ttk::checkbutton $w.f.cb -text [_ "Show this message again"] -variable TipWin::_mostrar($title)
	#            -command "TipWin::CambiaMostrar $title $w.f.cb"
	grid $w.f.cb -sticky we -padx 4 -pady 8
    }

    # marco de los botones
    frame $w.but -bd 0 -background [ CCColorActivo $def_back]
    ttk::button $w.but.close -text [_ "Close"] -command [list TipWin::Close $title] -takefocus 0
    SetWidgetsWidthFromText $w.but.close

    grid $w.but.close -sticky ews -padx 4 -pady 10

    #enpaquetamos marcos
    grid $w.f -sticky news -padx 0 -pady 0
    grid $w.but -sticky wes
    if { $::tcl_version >= 8.5 } { grid anchor $w.but center }

    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1

    AjustaPosicion $w $par

}


proc TipWin::CreateHelpButton { w title args} {
    ttk::button $w -image [GetImage help.gif] -command [list TipWin::CreateWindow . 0 $title $args]
}



# This file defines the procedure tk_optionMenuRAM, similar to
# the standard tk_optionMenu, but separating labels and commands
# to be used in multilingual codes.
# Creates an option button and its associated menu.
#
# ::tk_optionMenuRAM --
# This procedure creates an option button named $w and an associated
# menu.  Together they provide the functionality of Motif option menus:
# they can be used to select one of many values, and the current value
# appears in the global variable varName, as well as in the text of
# the option menubutton.  The name of the menu is returned as the
# procedure's result, so that the caller can use it to change configuration
# options on the menu or otherwise manipulate it.
#
# Arguments:
# w -                        The name to use for the menubutton.
# varname -                Global variable to hold the currently selected value.
# labels -                list of labels to show as menu options.
# values -                list of values (internal commands untranslated).



proc ::tk_optionMenuRAM {w varname labels values} {
  if { ![info exists ::$varname] } {
    set ::$varname [lindex $values 0]
  }

  #RAMSAN & KIKE
  if { [set idx [lsearch $values [set ::$varname]]] == -1 } {
    set idx 0
  }
  set ::GidPriv(tkopt,$w,text) [lindex $labels $idx]
  if { ![winfo exists $w] } {
    menubutton $w -text [lindex $labels $idx] -textvariable ::GidPriv(tkopt,$w,text) -indicatoron 0 -menu $w.menu \
	-bd 1 -highlightthickness 2 -anchor c \
	-direction flush -takefocus 1 -highlightthickness 1
    bind $w <Destroy> "unset ::GidPriv(tkopt,$w,text)"
  } else {
    $w conf -text [lindex $labels $idx] -textvariable ::GidPriv(tkopt,$w,text) -indicatoron 0 -menu $w.menu \
	-bd 2 -highlightthickness 2 -anchor c \
	-direction flush -takefocus "" -highlightthickness 1
  }
  menu $w.menu -tearoff 0
  foreach i $labels j $values {
    $w.menu add radiobutton -label $i -value $j -variable ::$varname  \
	-command "set ::GidPriv(tkopt,$w,text) [list $i]"
  }

  set cmd [list tk_optionMenuRAMselectlabel $labels $values ::GidPriv(tkopt,$w,text) ::$varname]
  trace add variable ::$varname write $cmd
  bind $w <Destroy> [list +trace remove variable ::$varname write $cmd]

  return $w.menu
}


proc tk_optionMenuRAMselectlabel { labels values menutextvariable fullvarname varname field args } {
    if { [info exists ::$fullvarname] } {
	set idx [lsearch $values [set ::$fullvarname]]
    } else {
	set idx -1
    }

    if { $idx != -1 } {
	set $menutextvariable [lindex $labels $idx]
    } else {
	#set $menutextvariable ""
	set $menutextvariable [set ::$fullvarname]
    }
}




namespace eval GiDSelectRowWindow {
    variable _w
    variable _list
    variable _seleccion
}

proc GiDSelectRowWindow::Close { } {
    variable _w

    destroy $_w
}

proc GiDSelectRowWindow::GetSelection { } {
    variable _list
    variable _seleccion

    set _seleccion [$_list curselection]
    if { "$_seleccion" != ""} {
	set _seleccion [ $_list get $_seleccion]
	GiDSelectRowWindow::Close
    } else {
	WarnWin [_ "Please, select first one sysinfo"] $_list
    }
}

# -np- GiDSelectRowWindow::Create { {{18 "type" left} {18 "description" left}} {"local machine" 123456789abcdef} {"usb key 1" aaaaaaaabbbbbbbb} {"usb key 2" 7878787812093467}} "Sysinfo selection" "Select one of the following sysinfos to register GiD"
# tabla datos: una lista de listas (filas) de listas(elementos de cada columna),
# donde la primera contiene el tamao titulo y justificacion para cada columna
proc GiDSelectRowWindow::Create { tabla_datos title msg { parent .gid}} {
    variable _w
    variable _list
    variable _seleccion


    package require tablelist_tile

    set _w $parent.row_window
    set w $_w

    catch { destroy $w }

    InitWindow $w $title GiDSelectRowWindowGeom
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0

    frame $w.f1

    set fila_tbl 0
    if { $msg != ""} {
	label $w.f1.l1 -text $msg -font BoldFont -justify left -anchor w -height 1 
	incr fila_tbl
    }

    set sw [ScrolledWindow $w.f1.lf -auto both -relief groove -borderwidth 2]

    set cols {}
    #
    set cab [ lindex $tabla_datos 0]
    foreach c $cab {
	foreach e $c {
	    lappend cols $e
	}
    }

    set _list [ tablelist::tablelist $sw.lb \
	    -exportselection 0 \
	    -columns $cols \
	    -labelcommand tablelist::sortByColumn \
	    -background white \
	    -selectbackground navy -selectforeground white \
	    -stretch end]
	    #-highlightthickness 0
    #-selectmode extended
    $sw setwidget $_list

    if { "$msg" != ""} {
	grid $w.f1.l1   -sticky ew
    }
    grid $sw       -sticky nsew
    grid columnconfigure $w.f1 0 -weight 1
    grid rowconfigure $w.f1 $fila_tbl -weight 1

    set def_back [$w cget -background]
    frame $w.buts -bg [CCColorActivo $def_back]

    ttk::button $w.buts.b1 -text [_ "Select"] -command [list GiDSelectRowWindow::GetSelection]
    ttk::button $w.buts.b2 -text [_ "Cancel"] -command [list GiDSelectRowWindow::Close]

    adjustButton $w.buts.b1
    adjustButton $w.buts.b2

    grid $w.buts.b1 $w.buts.b2 -padx 4 -pady 10

    grid $w.f1 -sticky nsew
    grid $w.buts -sticky ew
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1


    bind $w <Double-1> GiDSelectRowWindow::GetSelection

    update idletasks
    if { "$msg" != ""} {
	#$w.f1.l1 conf -wraplength [winfo reqwidth $w]
    }

    update

    # a rellenar datos
    $_list delete 0 end
    for { set i 1} { $i < [ llength $tabla_datos]} { incr i} {
	set f [ lindex $tabla_datos $i]
	$_list insert end $f
    }

    set _seleccion ""
    tkwait window $_w
    return $_seleccion

}


package require widgetsgid

#test to change use of tk_optionMenuRAM  by ComboBox
proc ::ComboBoxRAM {w varname labels values} {
    if { ![winfo exists $w] } {
	ComboBoxGiD $w -keyvariable $varname -values $labels -keyvalues $values ;#-valuesvariable ""
    } else {
	$w configure -keyvariable $varname -values $labels -keyvalues $values ;#-valuesvariable ""
    }
    if { ![info exists ::$varname] } {
	set ::$varname [lindex $values 0]
    } else {
	# set the variable to the same value, to refresh (with a trace) the ComboBoxGiD value
	set ::$varname [set ::$varname] ;#to refresh
    }
    return $w
}
