
proc pref_exchange_ui {base args} {
    global ChangeVarInfo

    ttk::labelframe $base.frame#1 -text [_ "Import"]
    ttk::labelframe $base.frame#2 -text [_ "Export"]
    ttk::frame $base.frame#3 -borderwidth 2 -relief groove
    ttk::frame $base.frame#4

    ttk::checkbutton $base.checkbutton#1 \
	-text [_ "Automatic collapse after import"] \
	-variable ChangeVarInfo(AutoCollapseAfterImport)

    GidHelp $base.checkbutton#1 [_ "If this option is set, after reading one\
	    IGES, DXF, VDA, Parasolid or ACIS file, one global collapse is made."]

    ttk::checkbutton $base.checkbutton#4 \
	-text [_ "Automatic import tolerance value"] \
	-variable ChangeVarInfo(AutoImportTolerance)

    GidHelp $base.checkbutton#4 [_ "If this option is set, the value of Import tolerance\
	    is assigned automatically according to the size of the model."]

    ttk::label $base.label#2 -text [_ "Import tolerance"]:

    ttk::entry $base.entry#9 -textvariable ChangeVarInfo(ImportTolerance)

    GidHelp "$base.label#2 $base.entry#9" [_ "When importing a file or\
	    collapsing, the points closer than this distance are considered\
	    to be unique. Lines and surfaces can also be collapsed."]

    ttk::label $base.label#3 -text [_ "Collapse"]:

    ttk::radiobutton $base.radiobutton#2 -text [_ "Ignoring layers"] \
	-variable ChangeVarInfo(CollapseIgnoringLayers) -value 1

    ttk::radiobutton $base.radiobutton#1 -text [_ "Each layer separately"] \
	-variable ChangeVarInfo(CollapseIgnoringLayers) -value 0

    GidHelp "$base.frame#3 $base.label#3  $base.radiobutton#2 \
	$base.radiobutton#1" [_ "if set to 'Ignoring Layers' entities\
	    are collapsed also if they belong to different layers. If set\
	    to 'Each layer separately' entities in different layers are not\
	    collapsed. (Entities belonging to frozen layers are never\
	    considered)."]

    ttk::checkbutton $base.checkbutton#5 \
	-text [_ "IGES: Curve on surface from 3D"] \
	-variable ChangeVarInfo(IGESCurveOnSurfaceFrom3D)

    GidHelp $base.checkbutton#5 [_ "If this option is set, the IGES curve on surface\
	    entities, are created from the direct 3D space definition (recommended),\
	    else are created from the surface space parameter definition."]

    ttk::checkbutton $base.checkbutton#6 \
	-text [_ "IGES: Create all in 'layer to use'"] \
	-variable ChangeVarInfo(IGESCreateAllInLayerToUse)

    GidHelp $base.checkbutton#6 [_ "If this option is set, the IGES entities, \
	    are created in the current layer to use, instead to use the file layers."]

    ttk::checkbutton $base.checkbutton#3 \
	-text [_ "IGES: B-Rep output style"] \
	-variable ChangeVarInfo(IGESSolidsManifoldBRep)

    GidHelp $base.checkbutton#3 [_ "If this option is set, then exported IGES entities\
	    are written with Boundary Representation Solid Model style, else the surfaces\
	    are written as separated trimmed surfaces, whitout topological information."]

    # Geometry management

    grid $base.frame#1 -sticky nsew -padx 2
    grid $base.frame#2 -sticky nsew -padx 2
    grid $base.checkbutton#1 -in $base.frame#1 -sticky w
    grid $base.checkbutton#4 -in $base.frame#1 -sticky w
    grid $base.frame#4 -in $base.frame#1 -sticky ew
    grid $base.frame#3 -in $base.frame#1 -sticky nsew -padx 2
    grid $base.checkbutton#5 -in $base.frame#1 -sticky w
    grid $base.checkbutton#6 -in $base.frame#1 -sticky w

    grid $base.label#2 $base.entry#9 -in $base.frame#4 -sticky w -padx 2
    grid configure $base.entry#9 -sticky ew -padx 2
    grid $base.label#3 -in $base.frame#3 -sticky w -padx 2
    grid $base.radiobutton#2 -in $base.frame#3 -sticky w
    grid $base.radiobutton#1 -in $base.frame#3 -sticky w
    grid $base.checkbutton#3 -in $base.frame#2  -sticky w
    #grid $base.checkbutton#5 -in $base.frame#2 -sticky w

    # Resize behavior management
    grid columnconfigure $base 0 -weight 1

    grid rowconfigure $base.frame#1 0 -weight 1
    grid columnconfigure $base.frame#1 0 -weight 1

    grid rowconfigure $base.frame#2 0 -weight 1
    grid columnconfigure $base.frame#2 0 -weight 1

    grid rowconfigure $base.frame#3 0 -weight 1
    grid columnconfigure $base.frame#3 1 -weight 1

    grid columnconfigure $base.frame#4 1 -weight 1

    global ChangeVarInfo
    set ChangeVarInfo(basei) $base

    trace variable ChangeVarInfo(AutoImportTolerance) w "AutoCollapseAfterImportProc ;#"
}

proc AutoCollapseAfterImportProc {} {
    global ChangeVarInfo
    if { $ChangeVarInfo(AutoImportTolerance) == 1 } {
	set ChangeVarInfo(ImportTolerance) [format %g [GiD_Info AutomaticTolerance]]
	if { [winfo exists $ChangeVarInfo(basei).entry#9] } {
	    $ChangeVarInfo(basei).entry#9 conf -state disabled
	}
    } else {
	set ChangeVarInfo(ImportTolerance) [format %g [GiD_Set ImportTolerance]]
	if { [winfo exists $ChangeVarInfo(basei).entry#9] } {
	    $ChangeVarInfo(basei).entry#9 conf -state normal
	}
    }
}

